@@ -21,6 +21,7 @@ package tech.libeufin.bank
 import kotlinx.serialization.Serializable
 import tech.libeufin.bank.db.Database
 import tech.libeufin.common.*
+import tech.libeufin.common.crypto.PwCrypto
 import tech.libeufin.common.db.DatabaseConfig
 import java.nio.file.Path
 import java.time.Duration
@@ -49,6 +50,7 @@ data class BankConfig(
     val tanChannels: Map<TanChannel, Pair<Path, Map<String, String>>>,
     val payto: BankPaytoCtx,
     val wireMethod: WireMethod,
+    val pwCrypto: PwCrypto,
     val gcAbortAfter: Duration,
     val gcCleanAfter: Duration,
     val gcDeleteAfter: Duration
@@ -119,6 +121,10 @@ private fun TalerConfig.loadBankConfig(): BankConfig = section("libeufin-bank").
         )
     }
 
+    val pwCrypto = map("pwd_hash_algorithm", "password hash algorithm", mapOf(
+        "bcrypt" to json<PwCrypto.Bcrypt>("pwd_hash_config", "bcrypt JSON config").require()
+    )).require()
+
     val ZERO = TalerAmount.zero(regionalCurrency)
     val MAX = TalerAmount.max(regionalCurrency)
     BankConfig(
@@ -144,13 +150,14 @@ private fun TalerConfig.loadBankConfig(): BankConfig = section("libeufin-bank").
         tanChannels = tanChannels,
         payto = payto,
         wireMethod = method,
+        pwCrypto = pwCrypto,
         gcAbortAfter = duration("gc_abort_after").require(),
         gcCleanAfter = duration("gc_clean_after").require(),
         gcDeleteAfter = duration("gc_delete_after").require(),
     )
 }
 
-fun TalerConfig.currencySpecificationFor(currency: String): CurrencySpecification
+private fun TalerConfig.currencySpecificationFor(currency: String): CurrencySpecification
     = sections.find {
         val section = section(it)
         it.startsWith("CURRENCY-") && section.boolean("enabled").require() && section.string("code").require() == currency
