@@ -124,6 +124,16 @@ sealed class Option<out T> {
             return Some(valueSerializer.deserialize(decoder))
         }
     }
+
+    companion object {
+        fun <T> invert(optional: Option<T>?): Option<T?> {
+            return when (optional) {
+                null -> Option.None
+                is Option.None -> Option.Some(null)
+                is Option.Some -> Option.Some(optional.value)
+            }
+        }
+    }
 }
 
 @Serializable
