@@ -28,10 +28,7 @@ import tech.libeufin.bank.auth.authAdmin
 import tech.libeufin.bank.db.ConversionDAO
 import tech.libeufin.bank.db.ConversionDAO.ConversionResult
 import tech.libeufin.bank.db.Database
-import tech.libeufin.common.TalerAmount
-import tech.libeufin.common.TalerErrorCode
-import tech.libeufin.common.apiError
-import tech.libeufin.common.conflict
+import tech.libeufin.common.*
 
 fun Routing.conversionApi(db: Database, ctx: BankConfig) = conditional(ctx.allowConversion) {
     get("/conversion-info/config") {
@@ -101,7 +98,7 @@ fun Routing.conversionApi(db: Database, ctx: BankConfig) = conditional(ctx.allow
             }
         }
     }
-    authAdmin(db, TokenScope.readwrite) {
+    authAdmin(db, ctx.pwCrypto, TokenScope.readwrite) {
         post("/conversion-info/conversion-rate") {
             val req = call.receive<ConversionRate>()
             for (regionalAmount in sequenceOf(req.cashin_fee, req.cashin_tiny_amount, req.cashout_min_amount)) {
@@ -110,6 +107,10 @@ fun Routing.conversionApi(db: Database, ctx: BankConfig) = conditional(ctx.allow
             for (fiatAmount in sequenceOf(req.cashout_fee, req.cashout_tiny_amount, req.cashin_min_amount)) {
                 ctx.checkFiatCurrency(fiatAmount)
             }
+            // cashout conversion tiny amount must >= 0.01 as it is what EBICS supports
+            if (req.cashout_tiny_amount.isSubCent()) {
+                throw badRequest("Sub-cent amounts no supported by cashout, cashout_tiny_amount must be >= 0.01")
+            }
             db.conversion.updateConfig(req)
             call.respond(HttpStatusCode.NoContent)
         }
