@@ -47,45 +47,45 @@ import java.util.*
 
 private val logger: Logger = LoggerFactory.getLogger("libeufin-bank-api")
 
-fun Routing.coreBankApi(db: Database, ctx: BankConfig) {
+fun Routing.coreBankApi(db: Database, cfg: BankConfig) {
     get("/config") {
         call.respond(
             Config(
-                bank_name = ctx.name,
-                base_url = ctx.baseUrl,
-                currency = ctx.regionalCurrency,
-                currency_specification = ctx.regionalCurrencySpec,
-                allow_conversion = ctx.allowConversion,
-                allow_registrations = ctx.allowRegistration,
-                allow_deletions = ctx.allowAccountDeletion,
-                default_debit_threshold = ctx.defaultDebtLimit,
-                supported_tan_channels = ctx.tanChannels.keys,
-                allow_edit_name = ctx.allowEditName,
-                allow_edit_cashout_payto_uri = ctx.allowEditCashout,
-                wire_type = ctx.wireMethod,
-                wire_transfer_fees = ctx.wireTransferFees,
-                min_wire_transfer_amount = ctx.minAmount,
-                max_wire_transfer_amount = ctx.maxAmount
+                bank_name = cfg.name,
+                base_url = cfg.baseUrl,
+                currency = cfg.regionalCurrency,
+                currency_specification = cfg.regionalCurrencySpec,
+                allow_conversion = cfg.allowConversion,
+                allow_registrations = cfg.allowRegistration,
+                allow_deletions = cfg.allowAccountDeletion,
+                default_debit_threshold = cfg.defaultDebtLimit,
+                supported_tan_channels = cfg.tanChannels.keys,
+                allow_edit_name = cfg.allowEditName,
+                allow_edit_cashout_payto_uri = cfg.allowEditCashout,
+                wire_type = cfg.wireMethod,
+                wire_transfer_fees = cfg.wireTransferFees,
+                min_wire_transfer_amount = cfg.minAmount,
+                max_wire_transfer_amount = cfg.maxAmount
             )
         )
     }
-    authAdmin(db, TokenScope.readonly) {
+    authAdmin(db, cfg.pwCrypto, TokenScope.readonly) {
         get("/monitor") {
             val params = MonitorParams.extract(call.request.queryParameters)
             call.respond(db.monitor(params))
         }
     }
-    coreBankTokenApi(db)
-    coreBankAccountsApi(db, ctx)
-    coreBankTransactionsApi(db, ctx)
-    coreBankWithdrawalApi(db, ctx)
-    coreBankCashoutApi(db, ctx)
-    coreBankTanApi(db, ctx)
+    coreBankTokenApi(db, cfg)
+    coreBankAccountsApi(db, cfg)
+    coreBankTransactionsApi(db, cfg)
+    coreBankWithdrawalApi(db, cfg)
+    coreBankCashoutApi(db, cfg)
+    coreBankTanApi(db, cfg)
 }
 
-private fun Routing.coreBankTokenApi(db: Database) {
+private fun Routing.coreBankTokenApi(db: Database, cfg: BankConfig) {
     val TOKEN_DEFAULT_DURATION: Duration = Duration.ofDays(1L)
-    auth(db, TokenScope.refreshable) {
+    auth(db, cfg.pwCrypto, TokenScope.refreshable) {
         post("/accounts/{USERNAME}/token") {
             val existingToken = call.authToken
             val req = call.receive<TokenRequest>()
@@ -111,7 +111,7 @@ private fun Routing.coreBankTokenApi(db: Database) {
                     Instant.MAX
                 } else {
                     try {
-                        logger.debug("Creating token with days duration: ${tokenDuration.toDays()}")
+                        logger.debug { "Creating token with days duration: ${tokenDuration.toDays()}" }
                         creationTime.plus(tokenDuration)
                     } catch (e: Exception) {
                         throw badRequest("Bad token duration: ${e.message}")
@@ -136,7 +136,7 @@ private fun Routing.coreBankTokenApi(db: Database) {
             )
         }
     }
-    auth(db, TokenScope.readonly) {
+    auth(db, cfg.pwCrypto, TokenScope.readonly) {
         delete("/accounts/{USERNAME}/token") {
             val token = call.authToken ?: throw badRequest("Basic auth not supported here.")
             db.token.delete(token)
@@ -223,7 +223,8 @@ suspend fun createAccount(
         tanChannel = req.tan_channel,
         checkPaytoIdempotent = req.payto_uri != null,
         ctx = cfg.payto,
-        minCashout = req.min_cashout
+        minCashout = req.min_cashout,
+        pwCrypto = cfg.pwCrypto
     )
 
     when (cfg.wireMethod) {
@@ -272,6 +273,12 @@ suspend fun patchAccount(
             "'admin' account cannot be public",
             TalerErrorCode.END
         )
+
+    if (login == "exchange" && req.is_taler_exchange == false)
+        throw conflict(
+            "'exchange' account must be a taler exchange account",
+            TalerErrorCode.END
+        )
     
     if (req.tan_channel is Option.Some && req.tan_channel.value != null && !cfg.tanChannels.contains(req.tan_channel.value)) {
         throw unsupportedTanChannel(req.tan_channel.value)
@@ -297,7 +304,7 @@ suspend fun patchAccount(
 }
 
 private fun Routing.coreBankAccountsApi(db: Database, ctx: BankConfig) {
-    authAdmin(db, TokenScope.readwrite, !ctx.allowRegistration) {
+    authAdmin(db, ctx.pwCrypto, TokenScope.readwrite, !ctx.allowRegistration) {
         post("/accounts") {
             val req = call.receive<RegisterAccountRequest>()
             when (val result = createAccount(db, ctx, req, isAdmin)) {
@@ -319,6 +326,7 @@ private fun Routing.coreBankAccountsApi(db: Database, ctx: BankConfig) {
     }
     auth(
         db,
+        ctx.pwCrypto, 
         TokenScope.readwrite,
         allowAdmin = true,
         requireAdmin = !ctx.allowAccountDeletion
@@ -349,7 +357,7 @@ private fun Routing.coreBankAccountsApi(db: Database, ctx: BankConfig) {
             }
         }
     }
-    auth(db, TokenScope.readwrite, allowAdmin = true) {
+    auth(db, ctx.pwCrypto, TokenScope.readwrite, allowAdmin = true) {
         patch("/accounts/{USERNAME}") {
             val (req, challenge) = call.receiveChallenge<AccountReconfiguration>(db, Operation.account_reconfig)
             val res = patchAccount(db, ctx, req, username, isAdmin, challenge != null, challenge?.channel, challenge?.info)
@@ -390,7 +398,7 @@ private fun Routing.coreBankAccountsApi(db: Database, ctx: BankConfig) {
                     TalerErrorCode.BANK_NON_ADMIN_PATCH_MISSING_OLD_PASSWORD
                 )
             }
-            when (db.account.reconfigPassword(username, req.new_password, req.old_password, isAdmin || challenge != null)) {
+            when (db.account.reconfigPassword(username, req.new_password, req.old_password, isAdmin || challenge != null, ctx.pwCrypto)) {
                 AccountPatchAuthResult.Success -> call.respond(HttpStatusCode.NoContent)
                 AccountPatchAuthResult.TanRequired -> call.respondChallenge(db, Operation.account_auth_reconfig, req)
                 AccountPatchAuthResult.UnknownAccount -> throw unknownAccount(username)
@@ -410,7 +418,7 @@ private fun Routing.coreBankAccountsApi(db: Database, ctx: BankConfig) {
             call.respond(PublicAccountsResponse(publicAccounts))
         }
     }
-    authAdmin(db, TokenScope.readonly) {
+    authAdmin(db, ctx.pwCrypto, TokenScope.readonly) {
         get("/accounts") {
             val params = AccountParams.extract(call.request.queryParameters)
             val accounts = db.account.pageAdmin(params, ctx.payto)
@@ -421,7 +429,7 @@ private fun Routing.coreBankAccountsApi(db: Database, ctx: BankConfig) {
             }
         }
     }
-    auth(db, TokenScope.readonly, allowAdmin = true) {
+    auth(db, ctx.pwCrypto, TokenScope.readonly, allowAdmin = true) {
         get("/accounts/{USERNAME}") {
             val account = db.account.get(username, ctx.payto) ?: throw unknownAccount(username)
             call.respond(account)
@@ -430,7 +438,7 @@ private fun Routing.coreBankAccountsApi(db: Database, ctx: BankConfig) {
 }
 
 private fun Routing.coreBankTransactionsApi(db: Database, ctx: BankConfig) {
-    auth(db, TokenScope.readonly, allowAdmin = true) {
+    auth(db, ctx.pwCrypto, TokenScope.readonly, allowAdmin = true) {
         get("/accounts/{USERNAME}/transactions") {
             val params = HistoryParams.extract(call.request.queryParameters)
             val bankAccount = call.bankInfo(db, ctx.payto)
@@ -446,13 +454,13 @@ private fun Routing.coreBankTransactionsApi(db: Database, ctx: BankConfig) {
         get("/accounts/{USERNAME}/transactions/{T_ID}") {
             val tId = call.longPath("T_ID")
             val tx = db.transaction.get(tId, username, ctx.payto) ?: throw notFound(
-                    "Bank transaction '$tId' not found",
-                    TalerErrorCode.BANK_TRANSACTION_NOT_FOUND
-                )
+                "Bank transaction '$tId' not found",
+                TalerErrorCode.BANK_TRANSACTION_NOT_FOUND
+            )
             call.respond(tx)
         }
     }
-    auth(db, TokenScope.readwrite) {
+    auth(db, ctx.pwCrypto, TokenScope.readwrite) {
         post("/accounts/{USERNAME}/transactions") {
             val (req, challenge) = call.receiveChallenge<TransactionCreateRequest>(db, Operation.bank_transaction)
 
@@ -506,7 +514,7 @@ private fun Routing.coreBankTransactionsApi(db: Database, ctx: BankConfig) {
 }
 
 private fun Routing.coreBankWithdrawalApi(db: Database, ctx: BankConfig) {
-    auth(db, TokenScope.readwrite) {
+    auth(db, ctx.pwCrypto, TokenScope.readwrite) {
         post("/accounts/{USERNAME}/withdrawals") {
             val req = call.receive<BankAccountCreateWithdrawalRequest>()
             req.amount?.run(ctx::checkRegionalCurrency)
@@ -614,7 +622,7 @@ private fun Routing.coreBankWithdrawalApi(db: Database, ctx: BankConfig) {
 }
 
 private fun Routing.coreBankCashoutApi(db: Database, ctx: BankConfig) = conditional(ctx.allowConversion) {
-    auth(db, TokenScope.readwrite) {
+    auth(db, ctx.pwCrypto, TokenScope.readwrite) {
         post("/accounts/{USERNAME}/cashouts") {
             val (req, challenge) = call.receiveChallenge<CashoutRequest>(db, Operation.cashout)
 
@@ -663,7 +671,7 @@ private fun Routing.coreBankCashoutApi(db: Database, ctx: BankConfig) = conditio
             }
         }
     }
-    auth(db, TokenScope.readonly, allowAdmin = true) {
+    auth(db, ctx.pwCrypto, TokenScope.readonly, allowAdmin = true) {
         get("/accounts/{USERNAME}/cashouts/{CASHOUT_ID}") {
             val id = call.longPath("CASHOUT_ID")
             val cashout = db.cashout.get(id, username) ?: throw notFound(
@@ -682,7 +690,7 @@ private fun Routing.coreBankCashoutApi(db: Database, ctx: BankConfig) = conditio
             }
         }
     }
-    authAdmin(db, TokenScope.readonly) {
+    authAdmin(db, ctx.pwCrypto, TokenScope.readonly) {
         get("/cashouts") {
             val params = PageParams.extract(call.request.queryParameters)
             val cashouts = db.cashout.pageAll(params)
@@ -696,7 +704,7 @@ private fun Routing.coreBankCashoutApi(db: Database, ctx: BankConfig) = conditio
 }
 
 private fun Routing.coreBankTanApi(db: Database, ctx: BankConfig) {
-    auth(db, TokenScope.readwrite) {
+    auth(db, ctx.pwCrypto, TokenScope.readwrite) {
         post("/accounts/{USERNAME}/challenge/{CHALLENGE_ID}") {
             val id = call.longPath("CHALLENGE_ID")
             val res = db.tan.send(
