@@ -32,7 +32,7 @@ import tech.libeufin.common.RevenueConfig
 import tech.libeufin.common.RevenueIncomingHistory
 
 fun Routing.revenueApi(db: Database, ctx: BankConfig) { 
-    auth(db, TokenScope.revenue) {
+    auth(db, ctx.pwCrypto, TokenScope.revenue) {
         get("/accounts/{USERNAME}/taler-revenue/config") {
             call.respond(RevenueConfig(
                 currency = ctx.regionalCurrency
