@@ -29,7 +29,7 @@ import tech.libeufin.common.getVersion
 class LibeufinBank : CliktCommand() {
     init {
         versionOption(getVersion())
-        subcommands(Serve(), DbInit(), CreateAccount(), EditAccount(), ChangePw(), GC(), CliConfigCmd(BANK_CONFIG_SOURCE))
+        subcommands(DbInit(), ChangePw(), Serve(), CreateAccount(), EditAccount(), GC(), BenchPwh(), CliConfigCmd(BANK_CONFIG_SOURCE))
     }
 
     override fun run() = Unit
