@@ -52,7 +52,8 @@ class AccountDAO(private val db: Database) {
         tanChannel: TanChannel?,
         // Whether to check [internalPaytoUri] for idempotency
         checkPaytoIdempotent: Boolean,
-        ctx: BankPaytoCtx
+        ctx: BankPaytoCtx,
+        pwCrypto: PwCrypto
     ): AccountCreationResult = db.serializableTransaction { conn ->
         val timestamp = Instant.now().micros()
         val idempotent = conn.withStatement("""
@@ -60,30 +61,39 @@ class AccountDAO(private val db: Database) {
                 AND email IS NOT DISTINCT FROM ?
                 AND phone IS NOT DISTINCT FROM ?
                 AND cashout_payto IS NOT DISTINCT FROM ?
+                AND tan_channel IS NOT DISTINCT FROM ?::tan_enum
                 AND (NOT ? OR internal_payto_uri=?)
                 AND is_public=?
                 AND is_taler_exchange=?
-                AND tan_channel IS NOT DISTINCT FROM ?::tan_enum
+                AND max_debt=(?,?)::taler_amount
+                AND ${if (minCashout == null) "min_cashout IS NULL" else "min_cashout IS NOT DISTINCT FROM (?,?)::taler_amount"}
                 ,internal_payto_uri, name
             FROM customers 
                 JOIN bank_accounts
                     ON customer_id=owning_customer_id
             WHERE login=?
         """) {
-            // TODO check max debt and min checkout ?
             setString(1, name)
             setString(2, email)
             setString(3, phone)
             setString(4, cashoutPayto?.full(name))
-            setBoolean(5, checkPaytoIdempotent)
-            setString(6, internalPayto.canonical)
-            setBoolean(7, isPublic)
-            setBoolean(8, isTalerExchange)
-            setString(9, tanChannel?.name)
-            setString(10, login)
+            setString(5, tanChannel?.name)
+            setBoolean(6, checkPaytoIdempotent)
+            setString(7, internalPayto.canonical)
+            setBoolean(8, isPublic)
+            setBoolean(9, isTalerExchange)
+            setLong(10, maxDebt.value)
+            setInt(11, maxDebt.frac)
+            if (minCashout != null) {
+                setLong(12, minCashout.value)
+                setInt(13, minCashout.frac)
+                setString(14, login)
+            } else {
+                setString(12, login)
+            }
             oneOrNull { 
                 Pair(
-                    PwCrypto.checkpw(password, it.getString(1)).match && it.getBoolean(2),
+                    pwCrypto.checkpw(password, it.getString(1)).match && it.getBoolean(2),
                     it.getBankPayto("internal_payto_uri", "name", ctx)
                 )
             } 
@@ -125,7 +135,7 @@ class AccountDAO(private val db: Database) {
             """
             ) {
                 setString(1, login)
-                setString(2, PwCrypto.hashpw(password))
+                setString(2, pwCrypto.hashpw(password))
                 setString(3, name)
                 setString(4, email)
                 setString(5, phone)
@@ -282,7 +292,7 @@ class AccountDAO(private val db: Database) {
             oneOrNull {
                 CurrentAccount(
                     id = it.getLong("customer_id"),
-                    channel = it.getString("tan_channel")?.run { TanChannel.valueOf(this) },
+                    channel = it.getOptEnum<TanChannel>("tan_channel"),
                     phone = it.getString("phone"),
                     email = it.getString("email"),
                     name = it.getString("name"),
@@ -411,7 +421,8 @@ class AccountDAO(private val db: Database) {
         login: String, 
         newPw: String, 
         oldPw: String?,
-        is2fa: Boolean
+        is2fa: Boolean,
+        pwCrypto: PwCrypto
     ): AccountPatchAuthResult = db.serializableTransaction { conn ->
         val (customerId, currentPwh, tanRequired) = conn.withStatement("""
             SELECT customer_id, password_hash, (NOT ? AND tan_channel IS NOT NULL) 
@@ -425,10 +436,10 @@ class AccountDAO(private val db: Database) {
         }
         if (tanRequired) {
             AccountPatchAuthResult.TanRequired
-        } else if (oldPw != null && !PwCrypto.checkpw(oldPw, currentPwh).match) {
+        } else if (oldPw != null && !pwCrypto.checkpw(oldPw, currentPwh).match) {
             AccountPatchAuthResult.OldPasswordMismatch
         } else {
-            val newPwh = PwCrypto.hashpw(newPw)
+            val newPwh = pwCrypto.hashpw(newPw)
             conn.withStatement("UPDATE customers SET password_hash=? WHERE customer_id=?") {
                 setString(1, newPwh)
                 setLong(2, customerId)
@@ -447,7 +458,7 @@ class AccountDAO(private val db: Database) {
     }
 
     /** Check password of account [login] against [pw], rehashing it if outdated  */
-    suspend fun checkPassword(login: String, pw: String): CheckPasswordResult {
+    suspend fun checkPassword(login: String, pw: String, pwCrypto: PwCrypto): CheckPasswordResult {
         // Get user current password hash
         val info = db.serializable(
             "SELECT customer_id, password_hash FROM customers WHERE login=? AND deleted_at IS NULL"
@@ -461,12 +472,12 @@ class AccountDAO(private val db: Database) {
         val (customerId, currentPwh) = info
 
         // Check password
-        val check = PwCrypto.checkpw(pw, currentPwh)
+        val check = pwCrypto.checkpw(pw, currentPwh)
         if (!check.match) return CheckPasswordResult.PasswordMismatch
 
         // Reshah if outdated
         if (check.outdated) {
-            val newPwh = PwCrypto.hashpw(pw)
+            val newPwh = pwCrypto.hashpw(pw)
             db.serializable(
                 "UPDATE customers SET password_hash=? where customer_id=? AND password_hash=?"
             ) { 
@@ -540,7 +551,7 @@ class AccountDAO(private val db: Database) {
                     email = Option.Some(it.getString("email")),
                     phone = Option.Some(it.getString("phone"))
                 ),
-                tan_channel = it.getString("tan_channel")?.run { TanChannel.valueOf(this) },
+                tan_channel = it.getOptEnum<TanChannel>("tan_channel"),
                 cashout_payto_uri = it.getString("cashout_payto"),
                 payto_uri = it.getBankPayto("internal_payto_uri", "name", ctx),
                 balance = Balance(
@@ -556,7 +567,7 @@ class AccountDAO(private val db: Database) {
                 min_cashout = it.getOptAmount("min_cashout", db.bankCurrency),
                 is_public = it.getBoolean("is_public"),
                 is_taler_exchange = it.getBoolean("is_taler_exchange"),
-                status = AccountStatus.valueOf(it.getString("status"))
+                status = it.getEnum("status")
             )
         }
     }
@@ -661,7 +672,7 @@ class AccountDAO(private val db: Database) {
                 is_public = it.getBoolean("is_public"),
                 is_taler_exchange = it.getBoolean("is_taler_exchange"),
                 payto_uri = it.getBankPayto("internal_payto_uri", "name", ctx),
-                status = AccountStatus.valueOf(it.getString("status"))
+                status = it.getEnum("status")
             )
         }
 }
\ No newline at end of file
