@@ -19,20 +19,14 @@
 
 package tech.libeufin.bank.db
 
-import kotlinx.coroutines.coroutineScope
 import kotlinx.coroutines.flow.Flow
-import kotlinx.coroutines.flow.first
-import kotlinx.coroutines.launch
-import kotlinx.coroutines.withTimeoutOrNull
 import org.slf4j.Logger
 import org.slf4j.LoggerFactory
 import tech.libeufin.bank.*
-import tech.libeufin.common.*
 import tech.libeufin.common.db.*
-import java.sql.PreparedStatement
-import java.sql.ResultSet
-import java.util.concurrent.ConcurrentHashMap
+import tech.libeufin.common.internalServerError
 import java.util.*
+import java.util.concurrent.ConcurrentHashMap
 
 private val logger: Logger = LoggerFactory.getLogger("libeufin-bank-db")
 
@@ -111,30 +105,31 @@ class Database(dbConfig: DatabaseConfig, internal val bankCurrency: String, inte
 
     suspend fun monitor(
         params: MonitorParams
-    ): MonitorResponse = conn { conn ->
-        val stmt = conn.prepareStatement("""
-            SELECT
-                cashin_count
-                ,(cashin_regional_volume).val as cashin_regional_volume_val
-                ,(cashin_regional_volume).frac as cashin_regional_volume_frac
-                ,(cashin_fiat_volume).val as cashin_fiat_volume_val
-                ,(cashin_fiat_volume).frac as cashin_fiat_volume_frac
-                ,cashout_count
-                ,(cashout_regional_volume).val as cashout_regional_volume_val
-                ,(cashout_regional_volume).frac as cashout_regional_volume_frac
-                ,(cashout_fiat_volume).val as cashout_fiat_volume_val
-                ,(cashout_fiat_volume).frac as cashout_fiat_volume_frac
-                ,taler_in_count
-                ,(taler_in_volume).val as taler_in_volume_val
-                ,(taler_in_volume).frac as taler_in_volume_frac
-                ,taler_out_count
-                ,(taler_out_volume).val as taler_out_volume_val
-                ,(taler_out_volume).frac as taler_out_volume_frac
-            FROM stats_get_frame(?::timestamp, ?::stat_timeframe_enum)
-        """)
-        stmt.setObject(1, params.date)
-        stmt.setString(2, params.timeframe.name)
-        stmt.oneOrNull {
+    ): MonitorResponse = serializable(
+        """
+        SELECT
+            cashin_count
+            ,(cashin_regional_volume).val as cashin_regional_volume_val
+            ,(cashin_regional_volume).frac as cashin_regional_volume_frac
+            ,(cashin_fiat_volume).val as cashin_fiat_volume_val
+            ,(cashin_fiat_volume).frac as cashin_fiat_volume_frac
+            ,cashout_count
+            ,(cashout_regional_volume).val as cashout_regional_volume_val
+            ,(cashout_regional_volume).frac as cashout_regional_volume_frac
+            ,(cashout_fiat_volume).val as cashout_fiat_volume_val
+            ,(cashout_fiat_volume).frac as cashout_fiat_volume_frac
+            ,taler_in_count
+            ,(taler_in_volume).val as taler_in_volume_val
+            ,(taler_in_volume).frac as taler_in_volume_frac
+            ,taler_out_count
+            ,(taler_out_volume).val as taler_out_volume_val
+            ,(taler_out_volume).frac as taler_out_volume_frac
+        FROM stats_get_frame(?::timestamp, ?::stat_timeframe_enum)
+        """
+    ) {
+        setObject(1, params.date)
+        setString(2, params.timeframe.name)
+        oneOrNull {
             fiatCurrency?.run {
                 MonitorWithConversion(
                     cashinCount = it.getLong("cashin_count"),
