@@ -21,8 +21,7 @@ package tech.libeufin.bank.db
 
 import tech.libeufin.bank.Operation
 import tech.libeufin.bank.TanChannel
-import tech.libeufin.common.db.one
-import tech.libeufin.common.db.oneOrNull
+import tech.libeufin.common.db.*
 import tech.libeufin.common.internalServerError
 import tech.libeufin.common.micros
 import java.time.Duration
@@ -85,9 +84,9 @@ class TanDAO(private val db: Database) {
         one {
             when {
                 it.getBoolean("out_no_op") -> TanSendResult.NotFound
-                else ->  TanSendResult.Success(
+                else -> TanSendResult.Success(
                     tanInfo = it.getString("out_tan_info"),
-                    tanChannel = it.getString("out_tan_channel").run { TanChannel.valueOf(this) },
+                    tanChannel = it.getEnum("out_tan_channel"),
                     tanCode = it.getString("out_tan_code")
                 )
             }
@@ -139,8 +138,8 @@ class TanDAO(private val db: Database) {
             when {
                 it.getBoolean("out_ok") -> TanSolveResult.Success(
                     body = it.getString("out_body"),
-                    op = Operation.valueOf(it.getString("out_op")),
-                    channel = it.getString("out_channel")?.run { TanChannel.valueOf(this) },
+                    op = it.getEnum("out_op"),
+                    channel = it.getOptEnum<TanChannel>("out_channel"),
                     info = it.getString("out_info")
                 )
                 it.getBoolean("out_no_op") -> TanSolveResult.NotFound
@@ -176,7 +175,7 @@ class TanDAO(private val db: Database) {
         oneOrNull { 
             Challenge(
                 body = it.getString("body"),
-                channel = it.getString("tan_channel")?.run { TanChannel.valueOf(this) },
+                channel = it.getOptEnum<TanChannel>("tan_channel"),
                 info = it.getString("tan_info")
             )
         }
