@@ -17,17 +17,18 @@
  * <http://www.gnu.org/licenses/>
  */
 
+import io.ktor.http.*
 import org.junit.Test
 import tech.libeufin.bank.db.TransactionDAO.BankTransactionResult
 import tech.libeufin.bank.db.WithdrawalDAO.WithdrawalCreationResult
 import tech.libeufin.common.*
-import tech.libeufin.common.db.one
+import tech.libeufin.common.db.*
 import java.time.Instant
 import java.util.*
 import kotlin.test.assertEquals
 
 class AmountTest {
-    // Test amount computation in database
+    // Test amount computation in db
     @Test
     fun computationTest() = bankSetup { db -> db.conn { conn ->
         conn.execSQLUpdate("UPDATE libeufin_bank.bank_accounts SET balance.val = 100000 WHERE internal_payto_uri = '${customerPayto.canonical}'")
@@ -38,50 +39,45 @@ class AmountTest {
                     ,max_debt = (?, ?)::taler_amount
             WHERE internal_payto_uri = '${merchantPayto.canonical}'
         """)
-        suspend fun routine(balance: TalerAmount, due: TalerAmount, hasBalanceDebt: Boolean, maxDebt: TalerAmount): Boolean {
+        suspend fun routine(
+            balance: TalerAmount,
+            hasDebt: Boolean,
+            maxDebt: TalerAmount,
+            amount: TalerAmount
+        ): Boolean {
             stmt.setLong(1, balance.value)
             stmt.setInt(2, balance.frac)
-            stmt.setBoolean(3, hasBalanceDebt)
+            stmt.setBoolean(3, hasDebt)
             stmt.setLong(4, maxDebt.value)
             stmt.setInt(5, maxDebt.frac)
 
             // Check bank transaction
             stmt.executeUpdate()
-            val txRes = db.transaction.create(
-                creditAccountPayto = customerPayto,
-                debitAccountUsername = "merchant",
-                subject = "test",
-                amount = due,
-                timestamp = Instant.now(),
-                is2fa = false,
-                requestUid = null,
-                wireTransferFees = TalerAmount.zero("KUDOS"),
-                minAmount = TalerAmount.zero("KUDOS"),
-                maxAmount = TalerAmount.max("KUDOS")
-            )
-            val txBool = when (txRes) {
-                BankTransactionResult.BalanceInsufficient -> false
-                is BankTransactionResult.Success -> true
+            val txRes = client.postA("/accounts/merchant/transactions") {
+                json {
+                    "payto_uri" to "$customerPayto?message="
+                    "amount" to amount
+                }
+            }
+            val txBool = when {
+                txRes.isStatus(HttpStatusCode.OK, null) -> true
+                txRes.isStatus(HttpStatusCode.Conflict, TalerErrorCode.BANK_UNALLOWED_DEBIT) -> false
                 else -> throw Exception("Unexpected error $txRes")
             }
 
             // Check whithdraw 
             stmt.executeUpdate()
-            for ((amount, suggested) in listOf(Pair(due, null), Pair(null, due), Pair(due, due))) {
-                val wRes = db.withdrawal.create(
-                    login = "merchant",
-                    uuid = UUID.randomUUID(),
-                    amount = due,
-                    suggested_amount = null,
-                    timestamp = Instant.now(),
-                    wireTransferFees = TalerAmount.zero("KUDOS"),
-                    minAmount = TalerAmount.zero("KUDOS"),
-                    maxAmount = TalerAmount.max("KUDOS")
-                )
-                val wBool = when (wRes) {
-                    WithdrawalCreationResult.BalanceInsufficient -> false
-                    WithdrawalCreationResult.Success -> true
-                    else -> throw Exception("Unexpected error $txRes")
+            for ((amount, suggested) in listOf(Pair(amount, null), Pair(null, amount), Pair(amount, amount))) {
+                val wRes = client.postA("/accounts/merchant/withdrawals") {
+                    json { 
+                        "amount" to amount
+                        "suggested_amount" to suggested
+                    } 
+                }
+                val wBool = when {
+                    wRes.isStatus(HttpStatusCode.OK, null) -> true
+                    wRes.isStatus(HttpStatusCode.Conflict, TalerErrorCode.BANK_UNALLOWED_DEBIT) -> false
+                    else -> throw Exception("Unexpected error $wRes")
                 }
                 // Logic must be the same
                 assertEquals(wBool, txBool)
@@ -93,44 +89,95 @@ class AmountTest {
         // Balance enough, assert for true
         assert(routine(
             balance = TalerAmount(10, 0, "KUDOS"),
-            due = TalerAmount(8, 0, "KUDOS"),
-            hasBalanceDebt = false,
-            maxDebt = TalerAmount(100, 0, "KUDOS")
+            hasDebt = false,
+            maxDebt = TalerAmount(100, 0, "KUDOS"),
+            amount = TalerAmount(8, 0, "KUDOS"),
         ))
         // Balance still sufficient, thanks for big enough debt permission.  Assert true.
         assert(routine(
             balance = TalerAmount(10, 0, "KUDOS"),
-            due = TalerAmount(80, 0, "KUDOS"),
-            hasBalanceDebt = false,
-            maxDebt = TalerAmount(100, 0, "KUDOS")
+            hasDebt = false,
+            maxDebt = TalerAmount(100, 0, "KUDOS"),
+            amount = TalerAmount(80, 0, "KUDOS"),
         ))
         // Balance not enough, max debt cannot cover, asserting for false.
         assert(!routine(
             balance = TalerAmount(10, 0, "KUDOS"),
-            due = TalerAmount(80, 0, "KUDOS"),
-            hasBalanceDebt = true,
-            maxDebt = TalerAmount(50, 0, "KUDOS")
+            hasDebt = true,
+            maxDebt = TalerAmount(50, 0, "KUDOS"),
+            amount = TalerAmount(80, 0, "KUDOS"),
         ))
         // Balance becomes enough, due to a larger max debt, asserting for true.
         assert(routine(
             balance = TalerAmount(10, 0, "KUDOS"),
-            due = TalerAmount(80, 0, "KUDOS"),
-            hasBalanceDebt = false,
-            maxDebt = TalerAmount(70, 0, "KUDOS")
+            hasDebt = false,
+            maxDebt = TalerAmount(70, 0, "KUDOS"),
+            amount = TalerAmount(80, 0, "KUDOS"),
         ))
         // Max debt not enough for the smallest fraction, asserting for false
         assert(!routine(
             balance = TalerAmount(0, 0, "KUDOS"),
-            due = TalerAmount(0, 2, "KUDOS"),
-            hasBalanceDebt = false,
-            maxDebt = TalerAmount(0, 1, "KUDOS")
+            hasDebt = false,
+            maxDebt = TalerAmount(0, 1, "KUDOS"),
+            amount = TalerAmount(0, 2, "KUDOS"),
         ))
         // Same as above, but already in debt.
         assert(!routine(
             balance = TalerAmount(0, 1, "KUDOS"),
-            due = TalerAmount(0, 1, "KUDOS"),
-            hasBalanceDebt = true,
-            maxDebt = TalerAmount(0, 1, "KUDOS")
+            hasDebt = true,
+            maxDebt = TalerAmount(0, 1, "KUDOS"),
+            amount = TalerAmount(0, 1, "KUDOS"),
+        ))
+    }}
+
+    // Max withdrawal amount computation in db
+    @Test
+    fun maxComputationTest() = bankSetup { db -> db.conn { conn ->
+        val update = conn.prepareStatement("""
+            UPDATE libeufin_bank.bank_accounts 
+                SET balance = (?, ?)::taler_amount
+                    ,has_debt = ?
+                    ,max_debt = (?, ?)::taler_amount
+            WHERE bank_account_id = 1
+        """)
+        val select = conn.prepareStatement("""
+            SELECT
+                (max_amount).val as max_amount_val
+                ,(max_amount).frac as max_amount_frac
+            FROM account_max_amount(1, (?, ?)::taler_amount) AS max_amount
+        """)
+        select.apply {
+            val max = TalerAmount.max("KUDOS")
+            setLong(1, max.value)
+            setInt(2, max.frac)
+        }
+        suspend fun routine(
+            balance: TalerAmount,
+            hasDebt: Boolean,
+            maxDebt: TalerAmount
+        ): TalerAmount {
+            update.apply {
+                setLong(1, balance.value)
+                setInt(2, balance.frac)
+                setBoolean(3, hasDebt)
+                setLong(4, maxDebt.value)
+                setInt(5, maxDebt.frac)
+                executeUpdate()
+            }
+            return select.one { it.getAmount("max_amount", "KUDOS") }
+        }
+
+        // Without debt
+        assertEquals(TalerAmount(110, 3, "KUDOS"), routine(
+            balance = TalerAmount(10, 1, "KUDOS"),
+            hasDebt = false,
+            maxDebt = TalerAmount(100, 2, "KUDOS"),
+        ))
+        // With debt
+        assertEquals(TalerAmount(90, 1, "KUDOS"), routine(
+            balance = TalerAmount(10, 1, "KUDOS"),
+            hasDebt = true,
+            maxDebt = TalerAmount(100, 2, "KUDOS"),
         ))
     }}
 
@@ -201,7 +248,7 @@ class AmountTest {
     fun conversionApply() = dbSetup { db ->
         db.conn { conn ->
             fun apply(nb: TalerAmount, times: DecimalNumber, tiny: DecimalNumber = DecimalNumber("0.00000001"), roundingMode: String = "zero"): TalerAmount {
-                val stmt = conn.prepareStatement("SELECT amount.val, amount.frac FROM conversion_apply_ratio((?, ?)::taler_amount, (?, ?)::taler_amount, (?, ?)::taler_amount, ?::rounding_mode) as amount")
+                val stmt = conn.prepareStatement("SELECT (result).val, (result).frac FROM conversion_apply_ratio((?, ?)::taler_amount, (?, ?)::taler_amount, (0, 0)::taler_amount, (?, ?)::taler_amount, ?::rounding_mode)")
                 stmt.setLong(1, nb.value)
                 stmt.setInt(2, nb.frac)
                 stmt.setLong(3, times.value)
@@ -264,16 +311,16 @@ class AmountTest {
     @Test
     fun conversionRevert() = dbSetup { db ->
         db.conn { conn ->
+            val applyStmt = conn.prepareStatement("SELECT (result).val, (result).frac FROM conversion_apply_ratio((?, ?)::taler_amount, (?, ?)::taler_amount, (0, 0)::taler_amount, (?, ?)::taler_amount, ?::rounding_mode)")
             fun TalerAmount.apply(ratio: DecimalNumber, tiny: DecimalNumber = DecimalNumber("0.00000001"), roundingMode: String = "zero"): TalerAmount {
-                val stmt = conn.prepareStatement("SELECT amount.val, amount.frac FROM conversion_apply_ratio((?, ?)::taler_amount, (?, ?)::taler_amount, (?, ?)::taler_amount, ?::rounding_mode) as amount")
-                stmt.setLong(1, this.value)
-                stmt.setInt(2, this.frac)
-                stmt.setLong(3, ratio.value)
-                stmt.setInt(4, ratio.frac)
-                stmt.setLong(5, tiny.value)
-                stmt.setInt(6, tiny.frac)
-                stmt.setString(7, roundingMode)
-                return stmt.one {
+                applyStmt.setLong(1, this.value)
+                applyStmt.setInt(2, this.frac)
+                applyStmt.setLong(3, ratio.value)
+                applyStmt.setInt(4, ratio.frac)
+                applyStmt.setLong(5, tiny.value)
+                applyStmt.setInt(6, tiny.frac)
+                applyStmt.setString(7, roundingMode)
+                return applyStmt.one {
                     TalerAmount(
                         it.getLong(1),
                         it.getInt(2),
@@ -282,16 +329,16 @@ class AmountTest {
                 }
             }
 
+            val revertStmt = conn.prepareStatement("SELECT (result).val, (result).frac FROM conversion_revert_ratio((?, ?)::taler_amount, (?, ?)::taler_amount, (0, 0)::taler_amount, (?, ?)::taler_amount, ?::rounding_mode)")
             fun TalerAmount.revert(ratio: DecimalNumber, tiny: DecimalNumber = DecimalNumber("0.00000001"), roundingMode: String = "zero"): TalerAmount {
-                val stmt = conn.prepareStatement("SELECT amount.val, amount.frac FROM conversion_revert_ratio((?, ?)::taler_amount, (?, ?)::taler_amount, (?, ?)::taler_amount, ?::rounding_mode) as amount")
-                stmt.setLong(1, this.value)
-                stmt.setInt(2, this.frac)
-                stmt.setLong(3, ratio.value)
-                stmt.setInt(4, ratio.frac)
-                stmt.setLong(5, tiny.value)
-                stmt.setInt(6, tiny.frac)
-                stmt.setString(7, roundingMode)
-                return stmt.one {
+                revertStmt.setLong(1, this.value)
+                revertStmt.setInt(2, this.frac)
+                revertStmt.setLong(3, ratio.value)
+                revertStmt.setInt(4, ratio.frac)
+                revertStmt.setLong(5, tiny.value)
+                revertStmt.setInt(6, tiny.frac)
+                revertStmt.setString(7, roundingMode)
+                return revertStmt.one {
                     TalerAmount(
                         it.getLong(1),
                         it.getInt(2),
@@ -302,22 +349,20 @@ class AmountTest {
     
             assertEquals(TalerAmount("EUR:6.41"), TalerAmount("EUR:30.0629").revert(DecimalNumber("4.69")))
             assertEquals(TalerAmount("EUR:6.41"), TalerAmount("EUR:6.41000641").revert(DecimalNumber("1.000001")))
-            assertEquals(TalerAmount("EUR:0.99999999"), TalerAmount("EUR:2.49999997").revert(DecimalNumber("2.5")))
+            assertEquals(TalerAmount("EUR:0.99999999"), TalerAmount("EUR:2.49999998").revert(DecimalNumber("2.5")))
             assertEquals(TalerAmount("EUR:${TalerAmount.MAX_VALUE}.99999999"), TalerAmount("EUR:${TalerAmount.MAX_VALUE}.99999999").revert(DecimalNumber("1")))
             assertEquals(TalerAmount("EUR:${TalerAmount.MAX_VALUE}"), TalerAmount("EUR:${TalerAmount.MAX_VALUE/4}").revert(DecimalNumber("0.25")))
             assertException("ERROR: amount value overflowed") { TalerAmount(TalerAmount.MAX_VALUE/4, 0, "EUR").revert(DecimalNumber("0.24999999")) }
             assertException("ERROR: amount value overflowed") { TalerAmount((TalerAmount.MAX_VALUE+2)/2, 0, "EUR").revert(DecimalNumber("0.5")) }
             assertException("ERROR: numeric field overflow") { TalerAmount(Long.MAX_VALUE, 0, "EUR").revert(DecimalNumber("1")) }
 
-            for (mode in listOf("zero", "up", "nearest")) {
-                for (amount in listOf(10, 11, 12, 12, 14, 15, 16, 17, 18, 19)) {
-                    for (tiny in listOf("0.01", "0.00000001", "5")) {
-                        for (ratio in listOf("1", "0.341", "0.00000001")) {
-                            val tiny = DecimalNumber(tiny)
-                            val ratio = DecimalNumber(ratio)
-                            val base = TalerAmount("EUR:$amount")
+
+            for (mode in sequenceOf("zero", "up", "nearest")) {
+                for (tiny in sequenceOf("0.01", "0.00000001", "1", "5").map(::DecimalNumber)) {
+                    for (amount in sequenceOf(10, 11, 12, 12, 14, 15, 16, 17, 18, 19).map { TalerAmount("EUR:$it") }) {
+                        for (ratio in sequenceOf("1", "0.01", "0.001", "0.00000001").map(::DecimalNumber)) {
                             // Apply ratio
-                            val rounded = base.apply(ratio, tiny, mode)
+                            val rounded = amount.apply(ratio, tiny, mode)
                             // Revert ratio  
                             val revert = rounded.revert(ratio, tiny, mode)
                             // Check applying ratio again give the same result
