@@ -769,6 +769,13 @@ class CoreBankAccountsApiTest {
             }
         }.assertConflict(TalerErrorCode.END)
 
+        // Exchange must be exchange
+        client.patchA("/accounts/exchange") {
+            json {
+                "is_taler_exchange" to false
+            }
+        }.assertConflict(TalerErrorCode.END)
+
         // Check cashout payto receiver name logic
         client.post("/accounts") {
             json {
@@ -961,7 +968,7 @@ class CoreBankAccountsApiTest {
             }
         }
         // All accounts
-        client.get("/accounts?delta=10"){
+        client.get("/accounts?limit=10"){
             pwAuth("admin")
         }.assertOkJson<ListBankAccountsResponse> {
             assertEquals(6, it.accounts.size)
