@@ -1,6 +1,6 @@
 /*
  * This file is part of LibEuFin.
- * Copyright (C) 2023 Taler Systems S.A.
+ * Copyright (C) 2023-2024 Taler Systems S.A.
 
  * LibEuFin is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Affero General Public License as
@@ -17,17 +17,24 @@
  * <http://www.gnu.org/licenses/>
  */
 
+import io.ktor.http.*
 import kotlinx.coroutines.coroutineScope
 import kotlinx.coroutines.launch
 import org.junit.Test
 import tech.libeufin.bank.createAdminAccount
 import tech.libeufin.bank.db.AccountDAO.AccountCreationResult
-import tech.libeufin.common.*
+import tech.libeufin.common.TalerError
+import tech.libeufin.common.TalerErrorCode
+import tech.libeufin.common.assertOk
+import tech.libeufin.common.db.one
+import tech.libeufin.common.db.oneOrNull
+import tech.libeufin.common.json
 import java.time.Duration
 import java.time.Instant
 import java.time.temporal.ChronoUnit
 import java.util.concurrent.TimeUnit
 import kotlin.test.assertEquals
+import kotlin.test.assertIs
 import kotlin.test.assertNull
 
 class DatabaseTest {
@@ -36,15 +43,17 @@ class DatabaseTest {
     @Test
     fun createAdmin() = setup { db, ctx ->
         // Create admin account
-        assertEquals(AccountCreationResult.Success, createAdminAccount(db, ctx))
+        assertIs<AccountCreationResult.Success>(createAdminAccount(db, ctx))
         // Checking idempotency
         assertEquals(AccountCreationResult.LoginReuse, createAdminAccount(db, ctx))
     }
 
     @Test
-    fun serialisation() = bankSetup {
+    fun serialisation() = bankSetup { db ->
         assertBalance("customer", "+KUDOS:0")
         assertBalance("merchant", "+KUDOS:0")
+
+        // Generate concurrent write transactions and check that they are all successful
         coroutineScope {
             repeat(10) { 
                 launch {
@@ -54,16 +63,50 @@ class DatabaseTest {
         }
         assertBalance("customer", "-KUDOS:4.5")
         assertBalance("merchant", "+KUDOS:4.5")
+
+        // Generate concurrent write and read transactions and check that they only write transaction sometimes fails
         coroutineScope {
-            repeat(5) { 
+            repeat(100) {
+                // Write transaction
                 launch {
-                    tx("customer", "KUDOS:0.0$it", "merchant", "concurrent 0$it")
+                    while (true) {
+                        val result = client.postA("/accounts/customer/transactions") {
+                            json {
+                                "payto_uri" to "$merchantPayto?message=${"concurrent 0$it".encodeURLQueryComponent()}&amount=KUDOS:0.0$it"
+                            }
+                        }
+                        if (result.status == HttpStatusCode.InternalServerError) {
+                            val body = result.json<TalerError>()
+                            assertEquals(TalerErrorCode.BANK_SOFT_EXCEPTION.code, body.code)
+                            continue // retry
+                        } else {
+                            result.assertOk()
+                            break
+                        }
+                    }
                 }
+                // Simple read transaction :SELECT
                 launch {
                     client.getA("/accounts/merchant/transactions").assertOk()
                 }
+                // Complex read transaction: stored procedure
+                launch {
+                    client.getA("/monitor") {
+                        pwAuth("admin")
+                    }.assertOk()
+                }
+                // GC logic
+                launch {
+                    try {
+                        db.gc.collect(Instant.now(), Duration.ofMinutes(1), Duration.ofMinutes(1), Duration.ofMinutes(1))
+                    } catch (e: Exception) {
+                        // Check only serialization exception
+                    }
+                }
             }
         }
+        assertBalance("customer", "-KUDOS:9.855")
+        assertBalance("merchant", "+KUDOS:9.855")
     }
 
     @Test
@@ -77,34 +120,34 @@ class DatabaseTest {
         val retransmissionPeriod: Duration = Duration.ofMinutes(1)
         val retryCounter = 3
 
-        fun create(code: String, now: Instant): Long {
+        fun create(code: String, timestamp: Instant): Long {
             createStmt.setString(1, code)
-            createStmt.setLong(2, ChronoUnit.MICROS.between(Instant.EPOCH, now))
+            createStmt.setLong(2, ChronoUnit.MICROS.between(Instant.EPOCH, timestamp))
             createStmt.setLong(3, TimeUnit.MICROSECONDS.convert(validityPeriod))
             createStmt.setInt(4, retryCounter)
-            return createStmt.oneOrNull { it.getLong(1) }!!
+            return createStmt.one { it.getLong(1) }
         }
 
-        fun markSent(id: Long, now: Instant) {
+        fun markSent(id: Long, timestamp: Instant) {
             markSentStmt.setLong(1, id)
-            markSentStmt.setLong(2, ChronoUnit.MICROS.between(Instant.EPOCH, now))
+            markSentStmt.setLong(2, ChronoUnit.MICROS.between(Instant.EPOCH, timestamp))
             markSentStmt.setLong(3, TimeUnit.MICROSECONDS.convert(retransmissionPeriod))
-            return markSentStmt.oneOrNull { }!!
+            return markSentStmt.one { }
         }
 
-        fun cTry(id: Long, code: String, now: Instant): Triple<Boolean, Boolean, Boolean> {
+        fun cTry(id: Long, code: String, timestamp: Instant): Triple<Boolean, Boolean, Boolean> {
             tryStmt.setLong(1, id)
             tryStmt.setString(2, code)
-            tryStmt.setLong(3, ChronoUnit.MICROS.between(Instant.EPOCH, now))
-            return tryStmt.oneOrNull { 
+            tryStmt.setLong(3, ChronoUnit.MICROS.between(Instant.EPOCH, timestamp))
+            return tryStmt.one { 
                 Triple(it.getBoolean(1), it.getBoolean(2), it.getBoolean(3))
-            }!!
+            }
         }
 
-        fun send(id: Long, code: String, now: Instant): String? {
+        fun send(id: Long, code: String, timestamp: Instant): String? {
             sendStmt.setLong(1, id)
             sendStmt.setString(2, code)
-            sendStmt.setLong(3, ChronoUnit.MICROS.between(Instant.EPOCH, now))
+            sendStmt.setLong(3, ChronoUnit.MICROS.between(Instant.EPOCH, timestamp))
             sendStmt.setLong(4, TimeUnit.MICROSECONDS.convert(validityPeriod))
             sendStmt.setInt(5, retryCounter)
             return sendStmt.oneOrNull {
