@@ -1,6 +1,6 @@
 /*
  * This file is part of LibEuFin.
- * Copyright (C) 2023 Taler Systems S.A.
+ * Copyright (C) 2023-2024 Taler Systems S.A.
 
  * LibEuFin is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Affero General Public License as
@@ -19,48 +19,71 @@
 
 import io.ktor.client.request.*
 import io.ktor.http.*
+import io.ktor.http.content.*
 import kotlinx.serialization.json.Json
 import org.junit.Test
-import tech.libeufin.common.TalerErrorCode
-import tech.libeufin.common.deflate
-import tech.libeufin.common.json
-import tech.libeufin.common.obj
+import tech.libeufin.common.*
 
 inline fun <reified B> HttpRequestBuilder.jsonDeflate(b: B) {
     val json = Json.encodeToString(kotlinx.serialization.serializer<B>(), b)
     contentType(ContentType.Application.Json)
-    headers.set(HttpHeaders.ContentEncoding, "deflate")
+    headers[HttpHeaders.ContentEncoding] = "deflate"
     setBody(json.toByteArray().inputStream().deflate().readBytes())
 }
 
+inline fun <reified B> HttpRequestBuilder.jsonStreamDeflate(b: B) {
+    val json = Json.encodeToString(kotlinx.serialization.serializer<B>(), b)
+    headers[HttpHeaders.ContentEncoding] = "deflate"
+    setBody(OutputStreamContent({
+        write(json.toByteArray().inputStream().deflate().readBytes())
+    }, ContentType.Application.Json))
+}
+
+inline fun <reified B> HttpRequestBuilder.jsonStream(b: B) {
+    val json = Json.encodeToString(kotlinx.serialization.serializer<B>(), b)
+    setBody(OutputStreamContent({
+        write(json.toByteArray())
+    }, ContentType.Application.Json))
+}
+
 class SecurityTest {
     @Test
-    fun bodySizeLimit() = bankSetup { _ ->
+    fun bodySizeLimit() = bankSetup {
         val valid_req = obj {
             "payto_uri" to "$exchangePayto?message=payout"
             "amount" to "KUDOS:0.3"
         }
+        val too_big = obj(valid_req) {
+            "payto_uri" to "$exchangePayto?message=payout${"A".repeat(MAX_BODY_LENGTH+1)}"
+        }
+
         client.postA("/accounts/merchant/transactions") {
             json(valid_req)
         }.assertOk()
-
+        
         // Check body too big
         client.postA("/accounts/merchant/transactions") {
-            json(valid_req) {
-                "payto_uri" to "$exchangePayto?message=payout${"A".repeat(4100)}"
-            }
+            json(too_big)
         }.assertBadRequest()
 
         // Check body too big even after compression
         client.postA("/accounts/merchant/transactions") {
-            jsonDeflate(obj(valid_req) {
-                "payto_uri" to "$exchangePayto?message=payout${"A".repeat(4100)}"
-            })
+            jsonDeflate(too_big)
+        }.assertBadRequest()
+
+        // Check streaming body too big
+        client.postA("/accounts/merchant/transactions") {
+            jsonStream(too_big)
+        }.assertBadRequest()
+
+        // Check streaming body too big even after compression
+        client.postA("/accounts/merchant/transactions") {
+            jsonStreamDeflate(too_big)
         }.assertBadRequest()
 
         // Check unknown encoding
         client.postA("/accounts/merchant/transactions") {
-            headers.set(HttpHeaders.ContentEncoding, "unknown")
+            headers[HttpHeaders.ContentEncoding] = "unknown"
             json(valid_req)
         }.assertStatus(HttpStatusCode.UnsupportedMediaType, TalerErrorCode.GENERIC_COMPRESSION_INVALID)
     }
