@@ -91,6 +91,10 @@ class StatsTest {
         monitorTalerIn(2, "KUDOS:10.6")
         addIncoming("KUDOS:12.3")
         monitorTalerIn(3, "KUDOS:22.9")
+
+        // KYC are ignored
+        addKyc("KUDOS:3")
+        monitorTalerIn(3, "KUDOS:22.9")
         
         transfer("KUDOS:10.0")
         monitorTalerOut(1, "KUDOS:10.0")
@@ -110,16 +114,16 @@ class StatsTest {
         monitorTalerIn(6, "KUDOS:78.84")
 
         cashout("KUDOS:3")
-        monitorCashout(1, "KUDOS:3", "EUR:3.747")
+        monitorCashout(1, "KUDOS:3", "EUR:3.74")
         cashout("KUDOS:7.6")
-        monitorCashout(2, "KUDOS:10.6", "EUR:13.244")
+        monitorCashout(2, "KUDOS:10.6", "EUR:13.23")
         cashout("KUDOS:12.3")
-        monitorCashout(3, "KUDOS:22.9", "EUR:28.616")
+        monitorCashout(3, "KUDOS:22.9", "EUR:28.6")
 
         monitorTalerIn(6, "KUDOS:78.84")
         monitorTalerOut(3, "KUDOS:82.5")
         monitorCashin(3, "KUDOS:55.94", "EUR:70")
-        monitorCashout(3, "KUDOS:22.9", "EUR:28.616")
+        monitorCashout(3, "KUDOS:22.9", "EUR:28.6")
     }
 
     @Test
