@@ -95,7 +95,8 @@ fun bankSetup(
         cashoutPayto = null,
         tanChannel = null,
         minCashout = null,
-        ctx = cfg.payto  
+        ctx = cfg.payto,
+        pwCrypto = cfg.pwCrypto
     ))
     assertIs<AccountCreationResult.Success>(db.account.create(
         login = "exchange",
@@ -112,7 +113,8 @@ fun bankSetup(
         cashoutPayto = null,
         tanChannel = null,
         minCashout = null,
-        ctx = cfg.payto   
+        ctx = cfg.payto,
+        pwCrypto = cfg.pwCrypto
     ))
     assertIs<AccountCreationResult.Success>(db.account.create(
         login = "customer",
@@ -129,7 +131,8 @@ fun bankSetup(
         cashoutPayto = null,
         tanChannel = null,
         minCashout = null,
-        ctx = cfg.payto
+        ctx = cfg.payto,
+        pwCrypto = cfg.pwCrypto
     ))
     // Create admin account
     assertIs<AccountCreationResult.Success>(createAdminAccount(db, cfg, "admin-password"))
@@ -149,7 +152,7 @@ fun bankSetup(
                     "cashin_min_amount" to "EUR:0"
                     "cashout_ratio" to "1.25"
                     "cashout_fee" to "EUR:0.003"
-                    "cashout_tiny_amount" to "EUR:0.00000001"
+                    "cashout_tiny_amount" to "EUR:0.01"
                     "cashout_rounding_mode" to "zero"
                     "cashout_min_amount" to "KUDOS:0.1"
                 }
@@ -228,6 +231,18 @@ suspend fun ApplicationTestBuilder.addIncoming(amount: String) {
     }.assertOk()
 }
 
+/** Perform a taler kyc transaction of [amount] from merchant to exchange */
+suspend fun ApplicationTestBuilder.addKyc(amount: String) {
+    client.post("/accounts/exchange/taler-wire-gateway/admin/add-kycauth") {
+        pwAuth("admin")
+        json {
+            "amount" to TalerAmount(amount)
+            "account_pub" to EddsaPublicKey.rand()
+            "debit_account" to merchantPayto
+        }
+    }.assertOk()
+}
+
 /** Perform a cashout operation of [amount] from customer */
 suspend fun ApplicationTestBuilder.cashout(amount: String) {
     val res = client.postA("/accounts/customer/cashouts") {
