@@ -201,12 +201,16 @@ class TalerAmount {
             repeat(8 - frac.length) {
                 tmp *= 10
             }
+            
             tmp
         }
     }
 
     fun number(): DecimalNumber = DecimalNumber(value, frac)
 
+    /* Check is amount has fractional amount < 0.01 */
+    fun isSubCent(): Boolean = (frac % CENT_FRACTION) > 0
+
     override fun equals(other: Any?): Boolean {
         return other is TalerAmount &&
                 other.value == this.value &&
@@ -238,6 +242,7 @@ class TalerAmount {
 
     companion object {
         const val FRACTION_BASE = 100000000
+        const val CENT_FRACTION = 1000000
         const val MAX_VALUE = 4503599627370496L // 2^52
         private val PATTERN = Regex("([A-Z]{1,11}):([0-9]+)(?:\\.([0-9]{1,8}))?")
 
