@@ -51,7 +51,7 @@ class TalerConfigError private constructor (m: String, cause: Throwable? = null)
 }
 
 /** Configuration error when converting option value */
-private class ValueError(val msg: String): Exception(msg)
+class ValueError(val msg: String): Exception(msg)
 
 /** Information about how the configuration is loaded */
 data class ConfigSource(
@@ -447,7 +447,7 @@ class TalerConfigSection internal constructor(
     val section: String
 ) {
     /** Setup an accessor/converted for a [type] at [option] using [transform] */
-    private fun <T> option(option: String, type: String, transform: TalerConfigSection.(String) -> T): TalerConfigOption<T> {
+    fun <T> option(option: String, type: String, transform: TalerConfigSection.(String) -> T): TalerConfigOption<T> {
         val canonOption = option.uppercase()
         var raw = entries?.get(canonOption)
         if (raw == "") raw = null
@@ -511,15 +511,18 @@ class TalerConfigSection internal constructor(
         }
     }
 
-    /** Access [option] as Map<String, String> */
-    fun jsonMap(option: String) = option(option, "json key/value map") {
+    /** Access [option] as JSON object [T] */
+    inline fun <reified T> json(option: String, type: String) = option(option, type) {
         try {
-            Json.decodeFromString<Map<String, String>>(it)
+            Json.decodeFromString<T>(it)
         } catch (e: Exception) {
             throw ValueError("'$it' is malformed")
         }
     }
 
+    /** Access [option] as Map<String, String> */
+    fun jsonMap(option: String) = json<Map<String, String>>(option, "json key/value map")
+
     /** Access [option] as TalerAmount */
     fun amount(option: String, currency: String) = option(option, "amount") {
         val amount = try {
