@@ -19,7 +19,20 @@
 
 package tech.libeufin.common
 
-import kotlinx.serialization.Serializable
+import kotlinx.serialization.*
+
+enum class TalerIncomingType {
+    reserve,
+    kyc,
+    wad
+}
+
+enum class TransferStatusState {
+    pending,
+    transient_failure,
+    permanent_failure,
+    success
+}
 
 /** Response GET /taler-wire-gateway/config */
 @Serializable
@@ -47,6 +60,33 @@ data class TransferResponse(
     val row_id: Long
 )
 
+/** Request GET /taler-wire-gateway/transfers */
+@Serializable
+data class TransferList(
+    val transfers: List<TransferListStatus>,
+    val debit_account: String
+)
+@Serializable
+data class TransferListStatus(
+    val row_id: Long,
+    val status: TransferStatusState,
+    val amount: TalerAmount,
+    val credit_account: String,
+    val timestamp: TalerProtocolTimestamp
+)
+
+/** Request GET /taler-wire-gateway/transfers/{ROW_iD} */
+@Serializable
+data class TransferStatus(
+    val status: TransferStatusState,
+    val status_msg: String? = null,
+    val amount: TalerAmount,
+    val origin_exchange_url: String,
+    val wtid: ShortHashCode,
+    val credit_account: String,
+    val timestamp: TalerProtocolTimestamp
+)
+
 /** Request POST /taler-wire-gateway/admin/add-incoming */
 @Serializable
 data class AddIncomingRequest(
@@ -62,23 +102,57 @@ data class AddIncomingResponse(
     val row_id: Long
 )
 
+/** Request POST /taler-wire-gateway/admin/add-kycauth */
+@Serializable
+data class AddKycauthRequest(
+    val amount: TalerAmount,
+    val account_pub: EddsaPublicKey,
+    val debit_account: Payto
+)
+
 /** Request GET /taler-wire-gateway/history/incoming */
 @Serializable
 data class IncomingHistory(
-    val incoming_transactions: List<IncomingReserveTransaction>,
+    val incoming_transactions: List<IncomingBankTransaction>,
     val credit_account: String
 )
 
 /** Inner request GET /taler-wire-gateway/history/incoming */
 @Serializable
+sealed interface IncomingBankTransaction {
+    val row_id: Long
+    val date: TalerProtocolTimestamp
+    val amount: TalerAmount
+    val debit_account: String
+};
+@Serializable
+@SerialName("KYCAUTH")
+data class IncomingKycAuthTransaction(
+    override val row_id: Long,
+    override val date: TalerProtocolTimestamp,
+    override val amount: TalerAmount,
+    override val debit_account: String,
+    val account_pub: EddsaPublicKey
+): IncomingBankTransaction
+@Serializable
+@SerialName("RESERVE")
 data class IncomingReserveTransaction(
-    val type: String = "RESERVE",
-    val row_id: Long, // DB row ID of the payment.
-    val date: TalerProtocolTimestamp,
-    val amount: TalerAmount,
-    val debit_account: String,
+    override val row_id: Long,
+    override val date: TalerProtocolTimestamp,
+    override val amount: TalerAmount,
+    override val debit_account: String,
     val reserve_pub: EddsaPublicKey
-)
+): IncomingBankTransaction
+@Serializable
+@SerialName("WAD")
+data class IncomingWadTransaction(
+    override val row_id: Long,
+    override val date: TalerProtocolTimestamp,
+    override val amount: TalerAmount,
+    override val debit_account: String,
+    val origin_exchange_url: String,
+    val wad_id: String // TODO 24 bytes Base32
+): IncomingBankTransaction
 
 /** Request GET /taler-wire-gateway/history/outgoing */
 @Serializable
