@@ -112,9 +112,9 @@ fun Application.talerApi(logger: Logger, routes: Routing.() -> Unit) {
             val path = call.request.path()
             val msg = call.logMsg()
             if (msg != null) {
-                "$status, $httpMethod $path, $msg"
+                "${status?.value} $httpMethod $path ${call.processingTimeMillis()}ms: $msg"
             } else {
-                "$status, $httpMethod $path"
+                "${status?.value} $httpMethod $path ${call.processingTimeMillis()}ms"
             }
         }
     }
