@@ -38,7 +38,7 @@ suspend fun <T> DbPool.page(
     bind: PreparedStatement.() -> Int = { 0 },
     map: (ResultSet) -> T
 ): List<T> {
-    val backward = params.delta < 0
+    val backward = params.limit < 0
     val pageQuery = """
         $query
         $idName ${if (backward) '<' else '>'} ?
@@ -47,8 +47,8 @@ suspend fun <T> DbPool.page(
     """
     return serializable(pageQuery) {
         val pad = bind()
-        setLong(pad + 1, params.start)
-        setInt(pad + 2, abs(params.delta))
+        setLong(pad + 1, params.offset)
+        setInt(pad + 2, abs(params.limit))
         all { map(it) }
     }
 }
@@ -76,17 +76,15 @@ suspend fun <T> DbPool.poolHistory(
         },
         map
     )
-        
-
-    // TODO do we want to handle polling when going backward and there is no transactions yet ?
+    
     // When going backward there is always at least one transaction or none
-    return if (params.page.delta >= 0 && params.polling.poll_ms > 0) {
+    return if (params.page.limit >= 0 && params.polling.timeout_ms > 0) {
         listen(bankAccountId) { flow ->
             coroutineScope {
                 // Start buffering notification before loading transactions to not miss any
                 val polling = launch {
-                    withTimeoutOrNull(params.polling.poll_ms) {
-                        flow.first { it > params.page.start } // Always forward so >
+                    withTimeoutOrNull(params.polling.timeout_ms) {
+                        flow.first { it > params.page.offset } // Always forward so >
                     }
                 }    
                 // Initial loading
@@ -127,17 +125,15 @@ suspend fun <T> DbPool.poolHistoryGlobal(
         query,
         map=map
     )
-        
 
-    // TODO do we want to handle polling when going backward and there is no transactions yet ?
     // When going backward there is always at least one transaction or none
-    return if (params.page.delta >= 0 && params.polling.poll_ms > 0) {
+    return if (params.page.limit >= 0 && params.polling.timeout_ms > 0) {
         listen { flow ->
             coroutineScope {
                 // Start buffering notification before loading transactions to not miss any
                 val polling = launch {
-                    withTimeoutOrNull(params.polling.poll_ms) {
-                        flow.first { it > params.page.start } // Always forward so >
+                    withTimeoutOrNull(params.polling.timeout_ms) {
+                        flow.first { it > params.page.offset } // Always forward so >
                     }
                 }    
                 // Initial loading
