@@ -19,14 +19,14 @@
 
 package tech.libeufin.common.db
 
-import kotlinx.coroutines.*
-import kotlinx.coroutines.flow.*
+import kotlinx.coroutines.delay
+import kotlinx.coroutines.flow.Flow
+import kotlinx.coroutines.flow.MutableSharedFlow
+import kotlinx.coroutines.runBlocking
 import org.postgresql.ds.PGSimpleDataSource
 import org.slf4j.Logger
-import org.slf4j.LoggerFactory
-import tech.libeufin.common.*
-import tech.libeufin.common.db.*
-import java.util.*
+import tech.libeufin.common.ExpoBackoffDecorr
+import tech.libeufin.common.fmtLog
 import java.util.concurrent.ConcurrentHashMap
 
 // SharedFlow that are manually counted for manual garbage collection
