@@ -22,6 +22,14 @@ package tech.libeufin.common.db
 import tech.libeufin.common.*
 import java.sql.ResultSet
 
+inline fun <reified T : kotlin.Enum<T>> ResultSet.getEnum(name: String): T 
+    = java.lang.Enum.valueOf(T::class.java, getString(name))
+inline fun <reified T : kotlin.Enum<T>> ResultSet.getEnum(idx: Int): T 
+    = java.lang.Enum.valueOf(T::class.java, getString(idx))
+
+inline fun <reified T : kotlin.Enum<T>> ResultSet.getOptEnum(name: String): T?
+    = getString(name)?.run { java.lang.Enum.valueOf(T::class.java, this) }
+
 fun ResultSet.getAmount(name: String, currency: String): TalerAmount {
     return TalerAmount(
         getLong("${name}_val"),
