@@ -44,11 +44,21 @@ class AmountTest {
 
     @Test
     fun parseRoundTrip() {
-        for (amount in listOf("EUR:4", "EUR:0.02", "EUR:4.12")) {
+        for (amount in sequenceOf("EUR:4", "EUR:0.02", "EUR:4.12")) {
             assertEquals(amount, TalerAmount(amount).toString())
         }
     }
 
+    @Test
+    fun subCent() {
+        for (ok in sequenceOf("EUR:1", "EUR:0.1", "EUR:0.01", "EUR:1.23")) {
+            assert(!TalerAmount(ok).isSubCent())
+        }
+        for (subCent in sequenceOf("EUR:0.001", "EUR:1.001", "EUR:99.991", "EUR:0.0000012")) {
+            assert(TalerAmount(subCent).isSubCent())
+        }
+    }
+
     fun assertException(msg: String, lambda: () -> Unit) {
         try {
             lambda()
