@@ -110,16 +110,21 @@ class CryptoUtilTest {
     @Test
     fun passwordHashing() {
         val password = "myinsecurepw"
-
+        val pwCrypto = PwCrypto.Bcrypt(cost = 4)
         // Check roundtrip
-        val hash = PwCrypto.hashpw(password)
-        assertEquals(PwCrypto.checkpw(password, hash), PasswordHashCheck(true, false))
-        assertEquals(PwCrypto.checkpw("other", hash), PasswordHashCheck(false, false))
+        val hash = pwCrypto.hashpw(password)
+        assertEquals(pwCrypto.checkpw(password, hash), PasswordHashCheck(true, false))
+        assertEquals(pwCrypto.checkpw("other", hash), PasswordHashCheck(false, false))
 
-        // Check outdated
+        // Check outdated algorithm
         val pwh = CryptoUtil.hashStringSHA256(password).encodeBase64()
         val outdatedHash = "sha256\$$pwh"
-        assertEquals(PwCrypto.checkpw(password, outdatedHash), PasswordHashCheck(true, true))
-        assertEquals(PwCrypto.checkpw("other", outdatedHash), PasswordHashCheck(false, true))
+        assertEquals(pwCrypto.checkpw(password, outdatedHash), PasswordHashCheck(true, true))
+        assertEquals(pwCrypto.checkpw("other", outdatedHash), PasswordHashCheck(false, true))
+
+        // Check outdated options
+        val betterCrypto = pwCrypto.copy(cost = 5)
+        assertEquals(betterCrypto.checkpw(password, hash), PasswordHashCheck(true, true))
+        assertEquals(betterCrypto.checkpw("other", hash), PasswordHashCheck(false, true))
     }
 }
