@@ -171,12 +171,15 @@ CREATE FUNCTION account_max_amount(
 LANGUAGE plpgsql AS $$
 BEGIN
 -- add balance and max_debt
-WITH added AS (
-  SELECT amount_add(balance, max_debt) as sum FROM bank_accounts
-    WHERE bank_account_id=in_account_id
-) SELECT (sum).val, (sum).frac
+WITH computed AS (
+  SELECT CASE has_debt
+    WHEN false THEN amount_add(balance, max_debt)
+    ELSE (SELECT diff FROM amount_left_minus_right(max_debt, balance))
+  END AS amount 
+  FROM bank_accounts WHERE bank_account_id=in_account_id
+) SELECT (amount).val, (amount).frac
   INTO out_max_amount.val, out_max_amount.frac
-  FROM added;
+  FROM computed;
 
 IF in_max_amount.val < out_max_amount.val 
   OR (in_max_amount.val = out_max_amount.val OR in_max_amount.frac < out_max_amount.frac) THEN
@@ -570,8 +573,10 @@ COMMENT ON PROCEDURE register_outgoing
   IS 'Register a bank transaction as a taler outgoing transaction and announce it';
 
 CREATE PROCEDURE register_incoming(
+  IN in_tx_row_id INT8,
+  IN in_type taler_incoming_type,
   IN in_reserve_pub BYTEA,
-  IN in_tx_row_id INT8
+  IN in_account_pub BYTEA
 )
 LANGUAGE plpgsql AS $$
 DECLARE
@@ -582,17 +587,24 @@ BEGIN
 INSERT
   INTO taler_exchange_incoming (
     reserve_pub,
-    bank_transaction
+    account_pub,
+    bank_transaction,
+    type
 ) VALUES (
   in_reserve_pub,
-  in_tx_row_id
+  in_account_pub,
+  in_tx_row_id,
+  in_type
 );
--- update stats
+-- Get bank transaction info
 SELECT (amount).val, (amount).frac, bank_account_id
 INTO local_amount.val, local_amount.frac, local_bank_account_id
 FROM bank_account_transactions WHERE bank_transaction_id=in_tx_row_id;
-CALL stats_register_payment('taler_in', NULL, local_amount, null);
--- notify new transaction
+-- Update stats
+IF in_type = 'reserve' THEN
+  CALL stats_register_payment('taler_in', NULL, local_amount, null);
+END IF;
+-- Notify new incoming transaction
 PERFORM pg_notify('incoming_tx', local_bank_account_id || ' ' || in_tx_row_id);
 END $$;
 COMMENT ON PROCEDURE register_incoming
@@ -688,12 +700,13 @@ END $$;
 COMMENT ON FUNCTION taler_transfer IS 'Create an outgoing taler transaction and register it';
 
 CREATE FUNCTION taler_add_incoming(
-  IN in_reserve_pub BYTEA,
+  IN in_key BYTEA,
   IN in_subject TEXT,
   IN in_amount taler_amount,
   IN in_debit_account_payto TEXT,
   IN in_username TEXT,
   IN in_timestamp INT8,
+  IN in_type taler_incoming_type,
   -- Error status
   OUT out_creditor_not_found BOOLEAN,
   OUT out_creditor_not_exchange BOOLEAN,
@@ -710,12 +723,13 @@ exchange_bank_account_id INT8;
 sender_bank_account_id INT8;
 BEGIN
 -- Check conflict
-SELECT true FROM taler_exchange_incoming WHERE reserve_pub = in_reserve_pub
-UNION ALL
-SELECT true FROM taler_withdrawal_operations WHERE reserve_pub = in_reserve_pub
-  INTO out_reserve_pub_reuse;
-IF out_reserve_pub_reuse THEN
-  RETURN;
+IF in_type = 'reserve'::taler_incoming_type THEN
+  SELECT EXISTS(SELECT FROM taler_exchange_incoming WHERE reserve_pub = in_key) OR
+    EXISTS(SELECT FROM taler_withdrawal_operations WHERE reserve_pub = in_key) 
+    INTO out_reserve_pub_reuse;
+  IF out_reserve_pub_reuse THEN
+    RETURN;
+  END IF;
 END IF;
 -- Find exchange bank account id
 SELECT
@@ -760,7 +774,11 @@ IF out_debitor_balance_insufficient THEN
   RETURN;
 END IF;
 -- Register incoming transaction
-CALL register_incoming(in_reserve_pub, out_tx_row_id);
+CASE in_type
+  WHEN 'reserve' THEN CALL register_incoming(out_tx_row_id, 'reserve', in_key, NULL);
+  WHEN 'kyc' THEN CALL register_incoming(out_tx_row_id, 'kyc', NULL, in_key);
+  ELSE RAISE EXCEPTION 'Unsupported incoming type %', in_type;
+END CASE;
 END $$;
 COMMENT ON FUNCTION taler_add_incoming IS 'Create an incoming taler transaction and register it';
 
@@ -968,9 +986,8 @@ END IF;
 
 IF not_selected THEN
   -- Check reserve_pub reuse
-  SELECT true FROM taler_exchange_incoming WHERE reserve_pub = in_reserve_pub
-  UNION ALL
-  SELECT true FROM taler_withdrawal_operations WHERE reserve_pub = in_reserve_pub
+  SELECT EXISTS(SELECT FROM taler_exchange_incoming WHERE reserve_pub = in_reserve_pub) OR
+    EXISTS(SELECT FROM taler_withdrawal_operations WHERE reserve_pub = in_reserve_pub) 
     INTO out_reserve_pub_reuse;
   IF out_reserve_pub_reuse THEN
     RETURN;
@@ -1126,7 +1143,7 @@ UPDATE taler_withdrawal_operations
   WHERE withdrawal_uuid=in_withdrawal_uuid;
 
 -- Register incoming transaction
-CALL register_incoming(reserve_pub_local, tx_row_id);
+CALL register_incoming(tx_row_id, 'reserve'::taler_incoming_type, reserve_pub_local, NULL);
 
 -- Notify status change
 PERFORM pg_notify('withdrawal_status', in_withdrawal_uuid::text || ' confirmed');
@@ -1204,7 +1221,7 @@ IF out_balance_insufficient THEN
 END IF;
 
 -- Register incoming transaction
-CALL register_incoming(in_reserve_pub, tx_row_id);
+CALL register_incoming(tx_row_id, 'reserve'::taler_incoming_type, in_reserve_pub, NULL);
 
 -- update stats
 CALL stats_register_payment('cashin', NULL, converted_amount, in_amount);
@@ -1617,33 +1634,43 @@ LANGUAGE sql AS $$ SELECT (value->>'mode')::rounding_mode FROM config WHERE key=
 CREATE FUNCTION conversion_apply_ratio(
    IN amount taler_amount
   ,IN ratio taler_amount
+  ,IN fee taler_amount
   ,IN tiny taler_amount       -- Result is rounded to this amount
   ,IN rounding rounding_mode  -- With this rounding mode
   ,OUT result taler_amount
+  ,OUT out_too_small BOOLEAN
 )
 LANGUAGE plpgsql AS $$
 DECLARE
-  product_numeric NUMERIC(33, 8); -- 16 digit for val, 8 for frac and 1 for rounding error
+  amount_numeric NUMERIC(33, 8); -- 16 digit for val, 8 for frac and 1 for rounding error
   tiny_numeric NUMERIC(24);
   rounding_error real;
 BEGIN
   -- Perform multiplication using big numbers
-  product_numeric = (amount.val::numeric(24) * 100000000 + amount.frac::numeric(24)) * (ratio.val::numeric(24, 8) + ratio.frac::numeric(24, 8) / 100000000);
+  amount_numeric = (amount.val::numeric(24) * 100000000 + amount.frac::numeric(24)) * (ratio.val::numeric(24, 8) + ratio.frac::numeric(24, 8) / 100000000);
+
+  -- Apply fees
+  amount_numeric = amount_numeric - (fee.val::numeric(24) * 100000000 + fee.frac::numeric(24));
+  IF (sign(amount_numeric) != 1) THEN
+    out_too_small = TRUE;
+    result = (0, 0);
+    RETURN;
+  END IF;
 
   -- Round to tiny amounts
   tiny_numeric = (tiny.val::numeric(24) * 100000000 + tiny.frac::numeric(24));
-  product_numeric = product_numeric / tiny_numeric;
-  rounding_error = (product_numeric % 1)::real;
-  product_numeric = trunc(product_numeric) * tiny_numeric;
+  amount_numeric = amount_numeric / tiny_numeric;
+  rounding_error = (amount_numeric % 1)::real;
+  amount_numeric = trunc(amount_numeric) * tiny_numeric;
   
   -- Apply rounding mode
   IF (rounding = 'nearest'::rounding_mode AND rounding_error >= 0.5)
     OR (rounding = 'up'::rounding_mode AND rounding_error > 0.0) THEN
-    product_numeric = product_numeric + tiny_numeric;
+    amount_numeric = amount_numeric + tiny_numeric;
   END IF;
 
   -- Extract product parts
-  result = (trunc(product_numeric / 100000000)::int8, (product_numeric % 100000000)::int4);
+  result = (trunc(amount_numeric / 100000000)::int8, (amount_numeric % 100000000)::int4);
 
   IF (result.val > 1::INT8<<52) THEN
     RAISE EXCEPTION 'amount value overflowed';
@@ -1655,32 +1682,37 @@ COMMENT ON FUNCTION conversion_apply_ratio
 CREATE FUNCTION conversion_revert_ratio(
    IN amount taler_amount
   ,IN ratio taler_amount
+  ,IN fee taler_amount
   ,IN tiny taler_amount       -- Result is rounded to this amount
   ,IN rounding rounding_mode  -- With this rounding mode
   ,OUT result taler_amount
+  ,OUT bad_value BOOLEAN
 )
 LANGUAGE plpgsql AS $$
 DECLARE
-  fraction_numeric NUMERIC(33, 8); -- 16 digit for val, 8 for frac and 1 for rounding error
+  amount_numeric NUMERIC(33, 8); -- 16 digit for val, 8 for frac and 1 for rounding error
   tiny_numeric NUMERIC(24);
   rounding_error real;
 BEGIN
+  -- Apply fees
+  amount_numeric = (amount.val::numeric(24) * 100000000 + amount.frac::numeric(24))  + (fee.val::numeric(24) * 100000000 + fee.frac::numeric(24));
+
   -- Perform division using big numbers
-  fraction_numeric = (amount.val::numeric(24) * 100000000 + amount.frac::numeric(24)) / (ratio.val::numeric(24, 8) + ratio.frac::numeric(24, 8) / 100000000);
+  amount_numeric = amount_numeric / (ratio.val::numeric(24, 8) + ratio.frac::numeric(24, 8) / 100000000);
 
   -- Round to tiny amounts
-  tiny_numeric = (tiny.val::numeric(24) * 100000000 + tiny.frac::numeric(24));
-  fraction_numeric = fraction_numeric / tiny_numeric;
-  rounding_error = (fraction_numeric % 1)::real;
-  fraction_numeric = trunc(fraction_numeric) * tiny_numeric;
+  tiny_numeric = (tiny.val::numeric(24) * 100000000 - tiny.frac::numeric(24));
+  amount_numeric = amount_numeric / tiny_numeric;
+  rounding_error = (amount_numeric % 1)::real;
+  amount_numeric = trunc(amount_numeric) * tiny_numeric;
 
-  -- Recover potentially lost tiny amount
-  IF (rounding = 'zero'::rounding_mode AND rounding_error > 0) THEN
-    fraction_numeric = fraction_numeric + tiny_numeric;
+  -- Recover potentially lost tiny amount during rounding
+  IF (rounding = 'zero'::rounding_mode AND rounding_error > 0.0) THEN
+    amount_numeric = amount_numeric + tiny_numeric;
   END IF;
 
   -- Extract division parts
-  result = (trunc(fraction_numeric / 100000000)::int8, (fraction_numeric % 100000000)::int4);
+  result = (trunc(amount_numeric / 100000000)::int8, (amount_numeric % 100000000)::int4);
 
   IF (result.val > 1::INT8<<52) THEN
     RAISE EXCEPTION 'amount value overflowed';
@@ -1730,12 +1762,8 @@ BEGIN
   SELECT value['val']::int8, value['frac']::int4 INTO tiny_amount.val, tiny_amount.frac FROM config WHERE key=direction||'_tiny_amount';
   SELECT (value->>'mode')::rounding_mode INTO mode FROM config WHERE key=direction||'_rounding_mode';
 
-  SELECT (diff).val, (diff).frac, NOT ok INTO converted.val, converted.frac, too_small 
-    FROM amount_left_minus_right(conversion_apply_ratio(amount, at_ratio, tiny_amount, mode), out_fee);
-
-  IF too_small THEN
-    converted = (0, 0);
-  END IF;
+  SELECT (result).val, (result).frac, out_too_small INTO converted.val, converted.frac, too_small 
+    FROM conversion_apply_ratio(amount, at_ratio, out_fee, tiny_amount, mode);
 END $$;
 
 CREATE FUNCTION conversion_from(
@@ -1764,8 +1792,8 @@ BEGIN
     no_config = true;
     RETURN;
   END IF;
-  SELECT result.val, result.frac INTO converted.val, converted.frac 
-    FROM conversion_revert_ratio(amount_add(amount, out_fee), at_ratio, tiny_amount, mode) as result;
+  SELECT (result).val, (result).frac INTO converted.val, converted.frac 
+    FROM conversion_revert_ratio(amount, at_ratio, out_fee, tiny_amount, mode);
   
   -- Check min amount
   SELECT value['val']::int8, value['frac']::int4 INTO min_amount.val, min_amount.frac FROM config WHERE key=direction||'_min_amount';
