@@ -8,7 +8,6 @@ LANGUAGE plpgsql AS $$
     now_date INT8;
     payto_uri TEXT;
   BEGIN
-    -- TODO should send to an exchange
     IF NEW.local_transaction IS NOT NULL THEN
       SELECT transaction_date INTO now_date
         FROM libeufin_bank.bank_account_transactions
@@ -50,6 +49,11 @@ LANGUAGE plpgsql AS $$
     no_account BOOLEAN;
     no_config BOOLEAN;
   BEGIN
+    -- Only reserve transaction triggers cashin
+    IF NEW.type != 'reserve' THEN
+      RETURN NEW;
+    END IF;
+
     SELECT (amount).val, (amount).frac, wire_transfer_subject, execution_time
       INTO local_amount.val, local_amount.frac, subject, now_date
       FROM libeufin_nexus.incoming_transactions
