@@ -28,22 +28,12 @@ import java.time.ZoneOffset
 import java.time.ZonedDateTime
 import java.time.format.DateTimeFormatter
 
-/**
- * Gets the amount number, also converting it from the
- * Taler-friendly 8 fractional digits to the more bank
- * friendly with 2.
- *
- * @param amount the Taler amount where to extract the number
- * @return [String] of the amount number without the currency.
- */
+/** String representation of a Taler [amount] compatible with EBICS */
 fun getAmountNoCurrency(amount: TalerAmount): String {
-    if (amount.frac == 0) {
-        return amount.value.toString()
-    } else {
-        val fractionFormat = amount.frac.toString().padStart(8, '0').dropLastWhile { it == '0' }
-        if (fractionFormat.length > 2) throw Exception("Sub-cent amounts not supported")
-        return "${amount.value}.${fractionFormat}"
+    if (amount.isSubCent()) {
+        throw Exception("Sub-cent amounts not supported")
     }
+    return amount.number().toString()
 }
 
 /** Create a pain.001 XML document valid for [dialect] */
@@ -369,23 +359,23 @@ fun parseTx(
         }
     }
     
+    /** Parse amount */
     fun XmlDestructor.amount(acceptedCurrency: String) = one("Amt") {
         val currency = attr("Ccy")
         /** FIXME: test by sending non-CHF to PoFi and see which currency gets here. */
         if (currency != acceptedCurrency) throw Exception("Currency $currency not supported")
         TalerAmount("$currency:${text()}")
     }
-
-    /** Check if transaction code is reversal */
-    fun XmlDestructor.isReversalCode(): Boolean {
+    
+    /** Parse bank transaction code */
+    fun XmlDestructor.bankTransactionCode(): BankTransactionCode {
         return one("BkTxCd").one("Domn") {
-            // TODO automate enum generation for all those code
-            val domainCode = one("Cd").text()
+            val domain = one("Cd").enum<ExternalBankTransactionDomainCode>()
             one("Fmly") {
-                val familyCode = one("Cd").text()
-                val subFamilyCode = one("SubFmlyCd").text()
+                val family = one("Cd").enum<ExternalBankTransactionFamilyCode>()
+                val subFamily = one("SubFmlyCd").enum<ExternalBankTransactionSubFamilyCode>()
                 
-                subFamilyCode == "RRTN" || subFamilyCode == "RPCR"
+                BankTransactionCode(domain, family, subFamily)
             }
         }
     }
@@ -406,17 +396,19 @@ fun parseTx(
                     val kind = one("CdtDbtInd").enum<Kind>()
                     val amount = amount(acceptedCurrency)
                     one("NtryDtls").one("TxDtls") { // TODO handle batches
+                        val code = bankTransactionCode()
+                        if (!code.isPayment()) return@one
                         val txRef = opt("Refs")?.opt("AcctSvcrRef")?.text()
-                        val reversal = isReversalCode()
                         val nexusId = nexusId()
-                        if (reversal) {
+                        if (code.isReversal()) {
                             if (kind == Kind.CRDT) {
                                 val reason = returnReason()
                                 txsInfo.add(TxInfo.CreditReversal(
                                     ref = nexusId ?: txRef ?: entryRef,
                                     bookDate = bookDate,
                                     nexusId = nexusId,
-                                    reason = reason
+                                    reason = reason,
+                                    code = code
                                 ))
                             }
                         } else {
@@ -431,7 +423,8 @@ fun parseTx(
                                         bankId = bankId,
                                         amount = amount,
                                         subject = subject,
-                                        debtorPayto = debtorPayto
+                                        debtorPayto = debtorPayto,
+                                        code = code
                                     ))
                                 }
                                 Kind.DBIT -> {
@@ -442,7 +435,8 @@ fun parseTx(
                                         nexusId = nexusId,
                                         amount = amount,
                                         subject = subject,
-                                        creditorPayto = creditorPayto
+                                        creditorPayto = creditorPayto,
+                                        code = code
                                     ))
                                 }
                             }
@@ -465,7 +459,8 @@ fun parseTx(
                 }
                 each("Ntry") {
                     if (!isBooked()) return@each
-                    if (isReversalCode()) return@each
+                    val code = bankTransactionCode()
+                    if (code.isReversal() || !code.isPayment()) return@each
                     val entryRef = opt("AcctSvcrRef")?.text()
                     val bookDate = executionDate()
                     val kind = one("CdtDbtInd").enum<Kind>()
@@ -483,7 +478,8 @@ fun parseTx(
                                 bankId = null,
                                 amount = amount,
                                 subject = subject,
-                                debtorPayto = debtorPayto
+                                debtorPayto = debtorPayto,
+                                code = code
                             ))
                         }
                     }
@@ -503,8 +499,9 @@ fun parseTx(
                 }
                 each("Ntry") {
                     if (!isBooked()) return@each
+                    val code = bankTransactionCode()
                     // Non reversal transaction are handled in camt.054
-                    if (!isReversalCode()) return@each
+                    if (!code.isReversal() || !code.isPayment()) return@each
 
                     val entryRef = opt("AcctSvcrRef")?.text()
                     val bookDate = executionDate()
@@ -518,7 +515,8 @@ fun parseTx(
                                 ref = nexusId ?: txRef ?: entryRef,
                                 bookDate = bookDate,
                                 nexusId = nexusId,
-                                reason = reason
+                                reason = reason,
+                                code = code
                             ))
                         }
                     }
@@ -531,8 +529,9 @@ fun parseTx(
                 }
                 each("Ntry") {
                     if (!isBooked()) return@each
+                    val code = bankTransactionCode()
                     // Reversal are handled from camt.053
-                    if (isReversalCode()) return@each
+                    if (code.isReversal() || !code.isPayment()) return@each
 
                     val entryRef = opt("AcctSvcrRef")?.text()
                     val bookDate = executionDate()
@@ -551,7 +550,8 @@ fun parseTx(
                                     bankId = bankId,
                                     amount = amount,
                                     subject = subject,
-                                    debtorPayto = debtorPayto
+                                    debtorPayto = debtorPayto,
+                                    code = code
                                 ))
                             }
                             Kind.DBIT -> {
@@ -563,7 +563,8 @@ fun parseTx(
                                     nexusId = nexusId,
                                     amount = amount,
                                     subject = subject,
-                                    creditorPayto = creditorPayto
+                                    creditorPayto = creditorPayto,
+                                    code = code
                                 ))
                             }
                         }
@@ -589,9 +590,12 @@ private sealed interface TxInfo {
     val ref: String?
     // When was this transaction booked
     val bookDate: Instant
+    // ISO20022 bank transaction code
+    val code: BankTransactionCode
     data class CreditReversal(
         override val ref: String?,
         override val bookDate: Instant,
+        override val code: BankTransactionCode,
         // Unique ID generated by libeufin-nexus
         val nexusId: String?,
         val reason: String?
@@ -599,6 +603,7 @@ private sealed interface TxInfo {
     data class Credit(
         override val ref: String?,
         override val bookDate: Instant,
+        override val code: BankTransactionCode,
         // Unique ID generated by payment provider
         val bankId: String?,
         val amount: TalerAmount,
@@ -608,6 +613,7 @@ private sealed interface TxInfo {
     data class Debit(
         override val ref: String?,
         override val bookDate: Instant,
+        override val code: BankTransactionCode,
         // Unique ID generated by libeufin-nexus
         val nexusId: String?,
         val amount: TalerAmount,
@@ -654,4 +660,23 @@ private fun parseTxLogic(info: TxInfo): TxNotification {
             )
         }
     }
+}
+
+data class BankTransactionCode(
+    val domain: ExternalBankTransactionDomainCode,
+    val family: ExternalBankTransactionFamilyCode,
+    val subFamily: ExternalBankTransactionSubFamilyCode
+) {
+    fun isReversal(): Boolean = REVERSAL_CODE.contains(subFamily)
+    fun isPayment(): Boolean = domain == ExternalBankTransactionDomainCode.PMNT
+
+    override fun toString(): String = 
+        "${domain.name} ${family.name} ${subFamily.name} - '${domain.description}' '${family.description}' '${subFamily.description}'"
+
+    companion object {
+        private val REVERSAL_CODE = setOf(
+            ExternalBankTransactionSubFamilyCode.RPCR,
+            ExternalBankTransactionSubFamilyCode.RRTN,
+        )
+    }
 }
\ No newline at end of file
