@@ -31,5 +31,5 @@ enum class HacAction(val description: String) {
 	VEU_CANCEL_ORDER("Cancellation of EDS order"),
 	ADDITIONAL("Additional information"),
 	ORDER_HAC_FINAL_POS("HAC end of order (positive)"),
-	ORDER_HAC_FINAL_NEG("ORDER_HAC_FINAL_NEG")
+	ORDER_HAC_FINAL_NEG("HAC end of order (negative)")
 }
\ No newline at end of file
