@@ -151,25 +151,25 @@ class XmlDestructor internal constructor(private val el: Element) {
         }.toList()
     }
 
-    fun one(path: String): XmlDestructor {
-        val children = el.childrenByTag(path).iterator()
+    fun one(tag: String): XmlDestructor {
+        val children = el.childrenByTag(tag).iterator()
         if (!children.hasNext()) {
-            throw DestructionError("expected unique '${el.tagName}.$path', got none")
+            throw DestructionError("expected unique '${el.tagName}.$tag', got none")
         }
         val el = children.next()
         if (children.hasNext()) {
-            throw DestructionError("expected unique '${el.tagName}.$path', got ${children.asSequence().count() + 1}")
+            throw DestructionError("expected unique '${el.tagName}.$tag', got ${children.asSequence().count() + 1}")
         }
         return XmlDestructor(el)
     }
-    fun opt(path: String): XmlDestructor? {
-        val children = el.childrenByTag(path).iterator()
+    fun opt(tag: String): XmlDestructor? {
+        val children = el.childrenByTag(tag).iterator()
         if (!children.hasNext()) {
             return null
         }
         val el = children.next()
         if (children.hasNext()) {
-            throw DestructionError("expected optional '${el.tagName}.$path', got ${children.asSequence().count() + 1}")
+            throw DestructionError("expected optional '${el.tagName}.$tag', got ${children.asSequence().count() + 1}")
         }
         return XmlDestructor(el)
     }
@@ -182,7 +182,7 @@ class XmlDestructor internal constructor(private val el: Element) {
     fun bool(): Boolean = el.textContent.toBoolean()
     fun date(): LocalDate = LocalDate.parse(text(), DateTimeFormatter.ISO_DATE)
     fun dateTime(): LocalDateTime = LocalDateTime.parse(text(), DateTimeFormatter.ISO_DATE_TIME)
-    inline fun <reified T : Enum<T>> enum(): T = java.lang.Enum.valueOf(T::class.java, text())
+    inline fun <reified T : kotlin.Enum<T>> enum(): T = java.lang.Enum.valueOf(T::class.java, text())
 
     fun attr(index: String): String = el.getAttribute(index)
 
