@@ -268,7 +268,7 @@ class EbicsBTS(
 
     private fun XmlBuilder.service(order: EbicsOrder.V3) {
         el("Service") {
-            el("ServiceName", order.name!!)
+            el("ServiceName", order.service!!)
             if (order.scope != null) {
                 el("Scope", order.scope)
             }
@@ -281,9 +281,9 @@ class EbicsBTS(
                 }
             }
             el("MsgName") {
-                if (order.messageVersion != null)
-                    attr("version", order.messageVersion)
-                text(order.messageName!!)
+                if (order.version != null)
+                    attr("version", order.version)
+                text(order.message!!)
             }
         }
     }
