@@ -67,7 +67,7 @@ const val grothoffPayto = "payto://iban/CH4189144589712575493?receiver-name=Grot
 
 val clientKeys = generateNewKeys()
 
-// Gets an HTTP client whose requests are going to be served by 'handler'.
+/** Gets an HTTP client whose requests are going to be served by 'handler' */
 fun getMockedClient(
     handler: MockRequestHandleScope.(HttpRequestData) -> HttpResponseData
 ): HttpClient = HttpClient(MockEngine) {
@@ -79,7 +79,7 @@ fun getMockedClient(
     }
 }
 
-// Generates a payment initiation, given its subject.
+/** Generates a payment initiation, given its subject */
 fun genInitPay(
     subject: String = "init payment",
     requestUid: String = "unique"
@@ -92,7 +92,7 @@ fun genInitPay(
         requestUid = requestUid
     )
 
-// Generates an incoming payment, given its subject.
+/** Generates an incoming payment, given its subject */
 fun genInPay(subject: String, amount: String = "KUDOS:44"): IncomingPayment {
     val bankId = run {
         val bytes = ByteArray(16).rand()
@@ -107,7 +107,7 @@ fun genInPay(subject: String, amount: String = "KUDOS:44"): IncomingPayment {
     )
 }
 
-// Generates an outgoing payment, given its subject and messageId
+/** Generates an outgoing payment, given its subject and messageId */
 fun genOutPay(subject: String, messageId: String? = null): OutgoingPayment {
     val id = messageId ?: run {
         val bytes = ByteArray(16).rand()
@@ -135,16 +135,50 @@ suspend fun ApplicationTestBuilder.transfer() {
     }.assertOk()
 }
 
+/** Perform a taler incoming transaction of [amount] from merchant to exchange */
+suspend fun ApplicationTestBuilder.addIncoming(amount: String) {
+    client.postA("/taler-wire-gateway/admin/add-incoming") {
+        json {
+            "amount" to TalerAmount(amount)
+            "reserve_pub" to EddsaPublicKey.rand()
+            "debit_account" to grothoffPayto
+        }
+    }.assertOk()
+}
+
+/** Perform a taler kyc transaction of [amount] from merchant to exchange */
+suspend fun ApplicationTestBuilder.addKyc(amount: String) {
+    client.postA("/taler-wire-gateway/admin/add-kycauth") {
+        json {
+            "amount" to TalerAmount(amount)
+            "account_pub" to EddsaPublicKey.rand()
+            "debit_account" to grothoffPayto
+        }
+    }.assertOk()
+}
+
 /** Ingest a talerable outgoing transaction */
 suspend fun talerableOut(db: Database) {
     val wtid = ShortHashCode.rand()
     ingestOutgoingPayment(db, genOutPay("$wtid http://exchange.example.com/"))
 }
 
-/** Ingest a talerable incoming transaction */
-suspend fun talerableIn(db: Database, nullId: Boolean = false) {
+/** Ingest a talerable reserve incoming transaction */
+suspend fun talerableIn(db: Database, nullId: Boolean = false, amount: String = "CHF:44") {
     val reserve_pub = ShortHashCode.rand()
-    ingestIncomingPayment(db, genInPay("history test with $reserve_pub reserve pub").run {
+    ingestIncomingPayment(db, genInPay("test with $reserve_pub reserve pub", amount).run {
+        if (nullId) {
+            copy(bankId = null)
+        } else {
+            this
+        }
+    }, AccountType.exchange)
+}
+
+/** Ingest a talerable KYC incoming transaction */
+suspend fun talerableKycIn(db: Database, nullId: Boolean = false, amount: String = "CHF:44") {
+    val account_pub = ShortHashCode.rand()
+    ingestIncomingPayment(db, genInPay("test with KYC:$account_pub account pub", amount).run {
         if (nullId) {
             copy(bankId = null)
         } else {
