@@ -90,10 +90,10 @@ class Cli : CliktCommand("Run integration tests on banks provider") {
         LIBEUFIN_NEXUS_HOME = test/$platform
 
         [nexus-fetch]
-        FREQUENCY = 5s
+        FREQUENCY = 5d
 
         [nexus-submit]
-        FREQUENCY = 5s
+        FREQUENCY = 5d
 
         [libeufin-nexusdb-postgres]
         CONFIG = postgres:///libeufintestbench
